/* ////////////////////////////////////////////////////////////////////////////
 * File:        comstl_interface_traits_std.h
 *
 * Purpose:     Standard interface traits.
 *
 * Created:     25th May 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS_STD
#define _COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS_STD

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_INTERFACE_TRAITS_STD_MAJOR     1
#define _COMSTL_VER_H_COMSTL_INTERFACE_TRAITS_STD_MINOR     2
#define _COMSTL_VER_H_COMSTL_INTERFACE_TRAITS_STD_REVISION  2
#define _COMSTL_VER_H_COMSTL_INTERFACE_TRAITS_STD_EDIT      18
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                    // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */
#ifndef _COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS
 #include "comstl_interface_traits.h"   // Include IID_traits
#endif /* !_COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup comstl_itf_library Interface Library
/// \ingroup COMSTL libraries
/// \brief This library provides facilities for manipulating COM interfaces
/// @{

/* ////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/* unknwn.idl */

#ifdef __IUnknown_FWD_DEFINED__
comstl_IID_traits_define(IUnknown)
#endif  /* __IUnknown_FWD_DEFINED__ */
#ifdef __IClassFactory_FWD_DEFINED__
comstl_IID_traits_define(IClassFactory)
#endif  /* __IClassFactory_FWD_DEFINED__ */

/* objidl.idl */
#ifdef __IMarshal_FWD_DEFINED__
comstl_IID_traits_define(IMarshal)
#endif  /* __IMarshal_FWD_DEFINED__ */
#ifdef __IMalloc_FWD_DEFINED__
comstl_IID_traits_define(IMalloc)
#endif  /* __IMalloc_FWD_DEFINED__ */
#ifdef __IMallocSpy_FWD_DEFINED__
comstl_IID_traits_define(IMallocSpy)
#endif  /* __IMallocSpy_FWD_DEFINED__ */
#ifdef __IStdMarshalInfo_FWD_DEFINED__
comstl_IID_traits_define(IStdMarshalInfo)
#endif  /* __IStdMarshalInfo_FWD_DEFINED__ */
#ifdef __IExternalConnection_FWD_DEFINED__
comstl_IID_traits_define(IExternalConnection)
#endif  /* __IExternalConnection_FWD_DEFINED__ */
#ifdef __IMultiQI_FWD_DEFINED__
comstl_IID_traits_define(IMultiQI)
#endif  /* __IMultiQI_FWD_DEFINED__ */
#ifdef __IEnumUnknown_FWD_DEFINED__
comstl_IID_traits_define(IEnumUnknown)
#endif  /* __IEnumUnknown_FWD_DEFINED__ */
#ifdef __IBindCtx_FWD_DEFINED__
comstl_IID_traits_define(IBindCtx)
#endif  /* __IBindCtx_FWD_DEFINED__ */
#ifdef __IEnumMoniker_FWD_DEFINED__
comstl_IID_traits_define(IEnumMoniker)
#endif  /* __IEnumMoniker_FWD_DEFINED__ */
#ifdef __IRunnableObject_FWD_DEFINED__
comstl_IID_traits_define(IRunnableObject)
#endif  /* __IRunnableObject_FWD_DEFINED__ */
#ifdef __IRunningObjectTable_FWD_DEFINED__
comstl_IID_traits_define(IRunningObjectTable)
#endif  /* __IRunningObjectTable_FWD_DEFINED__ */
#ifdef __IPersist_FWD_DEFINED__
comstl_IID_traits_define(IPersist)
#endif  /* __IPersist_FWD_DEFINED__ */
#ifdef __IPersistStream_FWD_DEFINED__
comstl_IID_traits_define(IPersistStream)
#endif  /* __IPersistStream_FWD_DEFINED__ */
#ifdef __IMoniker_FWD_DEFINED__
comstl_IID_traits_define(IMoniker)
#endif  /* __IMoniker_FWD_DEFINED__ */
#ifdef __IROTData_FWD_DEFINED__
comstl_IID_traits_define(IROTData)
#endif  /* __IROTData_FWD_DEFINED__ */
#ifdef __IEnumString_FWD_DEFINED__
comstl_IID_traits_define(IEnumString)
#endif  /* __IEnumString_FWD_DEFINED__ */
#ifdef __ISequentialStream_FWD_DEFINED__
comstl_IID_traits_define(ISequentialStream)
#endif  /* __ISequentialStream_FWD_DEFINED__ */
#ifdef __IStream_FWD_DEFINED__
comstl_IID_traits_define(IStream)
#endif  /* __IStream_FWD_DEFINED__ */
#ifdef __IEnumSTATSTG_FWD_DEFINED__
comstl_IID_traits_define(IEnumSTATSTG)
#endif  /* __IEnumSTATSTG_FWD_DEFINED__ */
#ifdef __IStorage_FWD_DEFINED__
comstl_IID_traits_define(IStorage)
#endif  /* __IStorage_FWD_DEFINED__ */
#ifdef __IPersistFile_FWD_DEFINED__
comstl_IID_traits_define(IPersistFile)
#endif  /* __IPersistFile_FWD_DEFINED__ */
#ifdef __IPersistStorage_FWD_DEFINED__
comstl_IID_traits_define(IPersistStorage)
#endif  /* __IPersistStorage_FWD_DEFINED__ */
#ifdef __ILockBytes_FWD_DEFINED__
comstl_IID_traits_define(ILockBytes)
#endif  /* __ILockBytes_FWD_DEFINED__ */
#ifdef __IEnumFORMATETC_FWD_DEFINED__
comstl_IID_traits_define(IEnumFORMATETC)
#endif  /* __IEnumFORMATETC_FWD_DEFINED__ */
#ifdef __IEnumSTATDATA_FWD_DEFINED__
comstl_IID_traits_define(IEnumSTATDATA)
#endif  /* __IEnumSTATDATA_FWD_DEFINED__ */
#ifdef __IRootStorage_FWD_DEFINED__
comstl_IID_traits_define(IRootStorage)
#endif  /* __IRootStorage_FWD_DEFINED__ */
#ifdef __IAdviseSink_FWD_DEFINED__
comstl_IID_traits_define(IAdviseSink)
#endif  /* __IAdviseSink_FWD_DEFINED__ */
#ifdef __IAdviseSink2_FWD_DEFINED__
comstl_IID_traits_define(IAdviseSink2)
#endif  /* __IAdviseSink2_FWD_DEFINED__ */
#ifdef __IDataObject_FWD_DEFINED__
comstl_IID_traits_define(IDataObject)
#endif  /* __IDataObject_FWD_DEFINED__ */
#ifdef __IDataAdviseHolder_FWD_DEFINED__
comstl_IID_traits_define(IDataAdviseHolder)
#endif  /* __IDataAdviseHolder_FWD_DEFINED__ */
#ifdef __IMessageFilter_FWD_DEFINED__
comstl_IID_traits_define(IMessageFilter)
#endif  /* __IMessageFilter_FWD_DEFINED__ */
#ifdef __IRpcChannelBuffer_FWD_DEFINED__
comstl_IID_traits_define(IRpcChannelBuffer)
#endif  /* __IRpcChannelBuffer_FWD_DEFINED__ */
#ifdef __IRpcProxyBuffer_FWD_DEFINED__
comstl_IID_traits_define(IRpcProxyBuffer)
#endif  /* __IRpcProxyBuffer_FWD_DEFINED__ */
#ifdef __IRpcStubBuffer_FWD_DEFINED__
comstl_IID_traits_define(IRpcStubBuffer)
#endif  /* __IRpcStubBuffer_FWD_DEFINED__ */
#ifdef __IPSFactoryBuffer_FWD_DEFINED__
comstl_IID_traits_define(IPSFactoryBuffer)
#endif  /* __IPSFactoryBuffer_FWD_DEFINED__ */
#ifdef __IChannelHook_FWD_DEFINED__
comstl_IID_traits_define(IChannelHook)
#endif  /* __IChannelHook_FWD_DEFINED__ */
#ifdef __IPropertyStorage_FWD_DEFINED__
comstl_IID_traits_define(IPropertyStorage)
#endif  /* __IPropertyStorage_FWD_DEFINED__ */
#ifdef __IPropertySetStorage_FWD_DEFINED__
comstl_IID_traits_define(IPropertySetStorage)
#endif  /* __IPropertySetStorage_FWD_DEFINED__ */
#ifdef __IEnumSTATPROPSTG_FWD_DEFINED__
comstl_IID_traits_define(IEnumSTATPROPSTG)
#endif  /* __IEnumSTATPROPSTG_FWD_DEFINED__ */
#ifdef __IEnumSTATPROPSETSTG_FWD_DEFINED__
comstl_IID_traits_define(IEnumSTATPROPSETSTG)
#endif  /* __IEnumSTATPROPSETSTG_FWD_DEFINED__ */
#ifdef __IClientSecurity_FWD_DEFINED__
comstl_IID_traits_define(IClientSecurity)
#endif  /* __IClientSecurity_FWD_DEFINED__ */
#ifdef __IServerSecurity_FWD_DEFINED__
comstl_IID_traits_define(IServerSecurity)
#endif  /* __IServerSecurity_FWD_DEFINED__ */
#ifdef __IClassActivator_FWD_DEFINED__
comstl_IID_traits_define(IClassActivator)
#endif  /* __IClassActivator_FWD_DEFINED__ */
#ifdef __IFillLockBytes_FWD_DEFINED__
comstl_IID_traits_define(IFillLockBytes)
#endif  /* __IFillLockBytes_FWD_DEFINED__ */
#ifdef __IProgressNotify_FWD_DEFINED__
comstl_IID_traits_define(IProgressNotify)
#endif  /* __IProgressNotify_FWD_DEFINED__ */
#ifdef __ILayoutStorage_FWD_DEFINED__
comstl_IID_traits_define(ILayoutStorage)
#endif  /* __ILayoutStorage_FWD_DEFINED__ */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_itf_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS_STD */

/* ////////////////////////////////////////////////////////////////////////// */
