/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_functions.h (formerly MWBase.h, ::SynesisWin)
 *
 * Purpose:     A number of useful functions .
 *
 * Created:     7th May 2000
 * Updated:     13th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_FUNCTIONS
#define _WINSTL_INCL_H_WINSTL_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_FUNCTIONS_MAJOR    1
#define _WINSTL_VER_H_WINSTL_FUNCTIONS_MINOR    4
#define _WINSTL_VER_H_WINSTL_FUNCTIONS_REVISION 2
#define _WINSTL_VER_H_WINSTL_FUNCTIONS_EDIT     8
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

#ifdef __cplusplus
inline bool BOOL2bool(BOOL b)
{
    return b != 0;
}
inline BOOL bool2BOOL(bool b)
{
    return b != false;
}
#else
 #define BOOL2bool(b)               SyCastStatic(bool, ((b) != 0))
 #define bool2BOOL(b)               SyCastStatic(BOOL, ((b) != false))
#endif /* __cplusplus */


/** CloseHandleSetNull
 *
 * Closes a handle, and sets its value to NULL
 */

inline ws_bool_t CloseHandleSetNull(HANDLE &h)
{
    return ::CloseHandle(h) ? (h = NULL, true) : false;
}

/** DuplicateLocalHandle()
 *
 * Duplicates a handle. Both the handle and the duplicate are within the context
 * of the current process.
 */
inline ws_bool_t DuplicateLocalHandle(HANDLE h, HANDLE &hDup)
{
    return BOOL2bool(::DuplicateHandle(::GetCurrentProcess(), h, ::GetCurrentProcess(), &hDup, 0, false, DUPLICATE_SAME_ACCESS));
}

/** FormatMessage()
 *
 */

template <ws_typename_param_k C>
inline C *__fmt_msg_find_last_good(C *first, C* last)
{
    for(;first != last; )
    {
        switch(*(last - 1))
        {
            case    ' ':
            case    '\t':
            case    '\r':
            case    '\n':
            case    '.':
                *(last - 1) = '\0';
                --last;
                break;
            default:
                first = last;
                break;
        }
    }

    return last;
}

inline ws_dword_t FormatMessage(DWORD dwErr, ws_char_a_t *buffer, ws_uint_t cchBuffer)
{
    ws_dword_t res = ::FormatMessageA(  FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
                                        NULL,
                                        dwErr,
                                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                                        buffer,
                                        cchBuffer,
                                        NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_a_t *last_good = __fmt_msg_find_last_good(buffer, buffer + res);

        *last_good = 0;
        res = last_good - buffer;

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD dwErr, ws_char_w_t *buffer, ws_uint_t cchBuffer)
{
    ws_dword_t res = ::FormatMessageW(  FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
                                        NULL,
                                        dwErr,
                                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                                        buffer,
                                        cchBuffer,
                                        NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_w_t *last_good = __fmt_msg_find_last_good(buffer, buffer + res);

        *last_good = 0;
        res = last_good - buffer;

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD dwErr, HINSTANCE hinstSource, ws_char_a_t **buffer)
{
    ws_dword_t  flags   =   FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_MAX_WIDTH_MASK | (NULL == hinstSource ? FORMAT_MESSAGE_FROM_SYSTEM : FORMAT_MESSAGE_FROM_HMODULE);
    ws_dword_t  res     =   ::FormatMessageA(flags, (void*)hinstSource, dwErr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)buffer, 0, NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_a_t *last_good = __fmt_msg_find_last_good(*buffer, *buffer + res);

        *last_good = 0;
        res = last_good - *buffer;

        winstl_assert(res >= 0);
    }

    return res;
}

inline ws_dword_t FormatMessage(DWORD dwErr, HINSTANCE hinstSource, ws_char_w_t **buffer)
{
    ws_dword_t  flags   =   FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_MAX_WIDTH_MASK | (NULL == hinstSource ? FORMAT_MESSAGE_FROM_SYSTEM : FORMAT_MESSAGE_FROM_HMODULE);
    ws_dword_t  res     =   ::FormatMessageW(flags, (void*)hinstSource, dwErr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPWSTR)buffer, 0, NULL);

    if(res != 0)
    {
        // Now trim the trailing space
        ws_char_w_t *last_good = __fmt_msg_find_last_good(*buffer, *buffer + res);

        *last_good = 0;
        res = last_good - *buffer;

        winstl_assert(res >= 0);
    }

    return res;
}



/** Elicits asynchronous key state
 */
inline ws_bool_t IsKeyPressedAsync(int k)
{
    return (::GetAsyncKeyState(k) & 0x8000) != 0;
}


/** Enables/disable a dialog item
 */
inline void EnableDlgItem(HWND hwnd, int id, ws_bool_t bEnable)
{
    ::EnableWindow(::GetDlgItem(hwnd, id), bEnable);
}

/** Elicits the enable status of a dialog item
 */
inline ws_bool_t IsDlgItemEnabled(HWND hwnd, int id)
{
    return ::IsWindowEnabled(::GetDlgItem(hwnd, id)) != 0;
}

/** Gets the text length of a dialog item's window contents
 */
inline int GetDlgItemTextLength(HWND hwnd, int id)
{
    return ::GetWindowTextLength(::GetDlgItem(hwnd, id));
}

/** Gets the HINSTANCE associated with a given window
 */
#ifdef GetWindowInstance
 #undef GetWindowInstance
#endif /* GetWindowInstance */
inline HINSTANCE GetWindowInstance(HWND hwnd)
{
    return (HINSTANCE)::GetWindowLong(hwnd, GWL_HINSTANCE);
}

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
