/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_setcursor_scope.h
 *
 * Purpose:     Cursor scoping class.
 *
 * Created:     12th May 2003
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_SETCURSOR_SCOPE
#define _WINSTL_INCL_H_WINSTL_SETCURSOR_SCOPE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_SETCURSOR_SCOPE_MAJOR      1
#define _WINSTL_VER_H_WINSTL_SETCURSOR_SCOPE_MINOR      0
#define _WINSTL_VER_H_WINSTL_SETCURSOR_SCOPE_REVISION   2
#define _WINSTL_VER_H_WINSTL_SETCURSOR_SCOPE_EDIT       7
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* _WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_STRING_ACCESS
 #include "stlsoft_string_access.h"     // stlsoft::c_str_ptr
#endif /* !_STLSOFT_INCL_H_STLSOFT_STRING_ACCESS */
#ifndef _WINSTL_INCL_H_WINSTL_STRING_ACCESS
 #include "winstl_string_access.h"      // stlsoft::c_str_ptr
#endif /* !_WINSTL_INCL_H_WINSTL_STRING_ACCESS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_window Window Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating GUI windows

/// \defgroup winstl_window_library Window Library (WinSTL)
/// \ingroup WinSTL libraries_window
/// \brief This library provides facilities for defining and manipulating Win32 GUI windows
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// setcursor_scope
/// Provides scoping of the enable status of a window.
///
/// This class provides scoping of the enable status of a window via the API
/// function EnableWindow()..
class setcursor_scope
{
public:
    typedef setcursor_scope class_type;

// Construction
public:
    /// \brief Toggles the current cursor
    ///
    /// Sets the given cursor.
    ws_explicit_k setcursor_scope(HCURSOR hcur)
        : m_hcurLast(::SetCursor(hcur))
    {}
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    /// \brief Toggles the current cursor
    ///
    /// Sets the given cursor resource from the given instance.
    template <ss_typename_param_k S>
    setcursor_scope(HINSTANCE hinst, S const &name)
        : m_hcurLast(::SetCursor(_load(hinst, c_str_ptr(name))))
    {}
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
    /// \brief Toggles the current cursor
    ///
    /// Sets the given cursor resource from the given instance.
    setcursor_scope(HINSTANCE hinst, ws_char_a_t const *name)
        : m_hcurLast(::SetCursor(_load(hinst, name)))
    {}
    /// \brief Toggles the current cursor
    ///
    /// Sets the given cursor resource from the given instance.
    setcursor_scope(HINSTANCE hinst, ws_char_w_t const *name)
        : m_hcurLast(::SetCursor(_load(hinst, name)))
    {}
    /// \brief Toggles the current cursor
    ///
    /// Sets the given cursor resource from the given instance.
    setcursor_scope(HINSTANCE hinst, int id)
        : m_hcurLast(::SetCursor(::LoadCursor(hinst, MAKEINTRESOURCE(id))))
    {}
    /// Resets the original cursor
    ~setcursor_scope()
    {
        ::SetCursor(m_hcurLast);
    }

/// Implementation
private:
    static HCURSOR _load(HINSTANCE hinst, ws_char_a_t const *name)
    {
        return ::LoadCursorA(hinst, name);
    }
    static HCURSOR _load(HINSTANCE hinst, ws_char_w_t const *name)
    {
        return ::LoadCursorW(hinst, name);
    }

/// Members
private:
    HCURSOR m_hcurLast;

/// Not to be implemented
private:
    setcursor_scope(setcursor_scope const &);
    setcursor_scope &operator =(setcursor_scope const &);
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_window_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_SETCURSOR_SCOPE */

/* ////////////////////////////////////////////////////////////////////////// */
