/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_windows_type_conversions.h (originally MWWinCmn.h, ::SynesisWin)
 *
 * Purpose:     Type conversions for Windows.
 *
 * Created:     
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS
#define _WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_MAJOR       1
#define _WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_MINOR       1
#define _WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_REVISION    2
#define _WINSTL_VER_H_WINSTL_WINDOWS_TYPE_CONVERSIONS_EDIT        10
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_SIMPLE_TYPE_CONVERTER
 #include "stlsoft_simple_type_converter.h" // stlsoft::simple_type_conversion
#endif /* !_STLSOFT_INCL_H_STLSOFT_SIMPLE_TYPE_CONVERTER */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

typedef stlsoft_ns_qual(simple_type_conversion)<char const *, LPARAM>       StrA2LPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<wchar_t const *, LPARAM>    StrW2LPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<TCHAR const *, LPARAM>      StrT2LPARAM;

typedef stlsoft_ns_qual(simple_type_conversion)<WPARAM, HWND>               WPARAM2HWND;
typedef stlsoft_ns_qual(simple_type_conversion)<LPARAM, HWND>               LPARAM2HWND;

typedef stlsoft_ns_qual(simple_type_conversion)<HWND, WPARAM>               HWND2WPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<HWND, LPARAM>               HWND2LPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<HWND, UINT>                 HWND2UINT;

typedef stlsoft_ns_qual(simple_type_conversion)<WPARAM, int>                WPARAM2int;

typedef stlsoft_ns_qual(simple_type_conversion)<WPARAM, HDROP>                WPARAM2HDROP;

typedef stlsoft_ns_qual(simple_type_conversion)<void*, WPARAM>              Ptr2WPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<void*, LPARAM>              Ptr2LPARAM;
typedef stlsoft_ns_qual(simple_type_conversion)<void*, UINT>                Ptr2UINT;

typedef stlsoft_ns_qual(simple_type_conversion)<int, char const *>          Id2StrA;
typedef stlsoft_ns_qual(simple_type_conversion)<int, wchar_t const *>       Id2StrW;
typedef stlsoft_ns_qual(simple_type_conversion)<int, TCHAR const *>         Id2StrT;

#if defined(_INC_COMMCTRL) || \
    defined(_COMMCTRL_H)
 typedef simple_type_conversion<LPARAM, LPNMHDR>                            LPARAM2LPNMHDR;
 typedef simple_type_conversion<LONG, HTREEITEM>                            LONG2HTREEITEM;
#endif /* _INC_COMMCTRL || _COMMCTRL_H */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_WINDOWS_TYPE_CONVERSIONS */

/* ////////////////////////////////////////////////////////////////////////// */
