/* /////////////////////////////////////////////////////////////////////////////
 * File:        rdempty.cpp
 *
 * Purpose:     Implementation file for the rdempty project.
 *
 * Created:     21st August 2003
 * Updated:     22nd August 2003
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <windows.h>
#include <stdio.h>

#include <stlsoft.h>

#include <winstl.h>

#include <winstl_findfile_sequence.h>

winstl_ns_using(findfile_sequence_a)

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _SYNSOFT_INTERNAL_BUILD
 #include <MLCompId.h>					// The _xxxSynesisSoftware constants
#else
 #define _nameSynesisSoftware				"Synesis Software (Pty) Ltd"
 #define _wwwSynesisSoftware_SystemTools	"http://synesis.com.au/"
 #define _emailSynesisSoftware_SystemTools	"software@synesis.com.au"
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 *
 * Normally, Synesis Software components & tools are "versioned" by making use
 * of tool-generated header files, one of which - MBldHdr.h - defines symbolic
 * constants representing major, minor, revision and build numbers.
 *
 * Since this file is to be available in a more public forum, including on the
 * Digital Mars site, these constants are merely defined here, rather than
 * encumber any developers who may wish to compile it with a lot of
 * Synesis-specific gunk.
 */

#ifdef _SYNSOFT_INTERNAL_BUILD
 #include "MBldHdr.h"

 const int	_mccVerHi	=	__SYV_MAJOR;
 const int	_mccVerLo	=	__SYV_MINOR;
 const int	_mccVerRev	=	__SYV_REVISION;
 const int	_mccBldNum	=	__SYV_BUILDNUMBER;

#else
 const int	_mccVerHi	=	1;
 const int	_mccVerLo	=	0;
 const int	_mccVerRev	=	2;
 const int	_mccBldNum	=	3;
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit);
static bool directory_is_empty(char const *dir);
static int empty_directory(char const *dir);

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char *argv[])
{
    int         iRet    =   0;
    int         i;
	char const	*root	=	NULL;
	char		sz[1 + _MAX_PATH];
	LPTSTR		p;

#ifdef _DEBUG
	puts("rdempty: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* _DEBUG */

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			usage(true);

			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
		}
	}

	/* . */
	if(NULL == root)
	{
		root = ".";
	}

	if(!::GetFullPathName(root, stlsoft_num_elements(sz), sz, &p))
	{
		printf("Directory \"%s\" does not exist!\n", root);
	}
	else
	{
		root = sz;

		printf("Cleaning directory \"%s\"\n", root);

		int	cRemoved	=	empty_directory(root);

		printf("Removed %d empty directories\n", cRemoved);
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

bool directory_is_empty(char const *dir)
{
	return findfile_sequence_a(dir, "*.*", findfile_sequence_a::files | findfile_sequence_a:: directories).size() == 0;
}

int empty_directory(char const *dir)
{
	int	cRemoved	=	0;

	findfile_sequence_a					dirs(dir, "*.*", findfile_sequence_a::directories);
	findfile_sequence_a::const_iterator	begin = dirs.begin();
	findfile_sequence_a::const_iterator	end		= dirs.end();

	for(; begin != end; ++begin)
	{
		cRemoved += empty_directory((*begin).get_path());
	}

	if(directory_is_empty(dir))
	{
		if(::RemoveDirectory(dir))
		{
			++cRemoved;

			printf("Removed \"%s\"\n", dir);
		}
		else
		{
			printf("Could not remove \"%s\"\n", dir);
		}
	}

	return cRemoved;
}

void usage(int bExit)
{
    fprintf(stderr, "\n");
    fprintf(stderr, "  Synesis Software rdempty v%d.%d.%d.%04d\n\n", _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stderr, "  USAGE: rdempty [<dir>]\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    <dir>          - root directory in search (defaults to current directory)\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");

    if(bExit)
    {
        exit(1);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
