/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_frame_array.h (originally MTArray.h, ::SynesisStl)
 *
 * Purpose:     Contains the frame_array template class.
 *
 * Created:     4th August 1998
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_FRAME_ARRAY
#define _STLSOFT_INCL_H_STLSOFT_FRAME_ARRAY

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_FRAME_ARRAY_MAJOR     1
#define _STLSOFT_VER_H_STLSOFT_FRAME_ARRAY_MINOR     8
#define _STLSOFT_VER_H_STLSOFT_FRAME_ARRAY_REVISION  7
#define _STLSOFT_VER_H_STLSOFT_FRAME_ARRAY_EDIT      111
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"                   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_NULL_ALLOCATOR
 #include "stlsoft_null_allocator.h"    // stlsoft::null_allocator
#endif /* !_STLSOFT_INCL_H_STLSOFT_NULL_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"          // stlsoft::iterator, stlsoft::reverse_iterator
#endif /* _STLSOFT_INCL_H_STLSOFT_ITERATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_ARRAY_POLICIES
 #include "stlsoft_array_policies.h"    // stlsoft::do_construction
#endif /* !_STLSOFT_INCL_H_STLSOFT_ARRAY_POLICIES */
#if defined(__STLSOFT_COMPILER_IS_MSVC) && \
    defined(__STLSOFT_CF_STL_IS_STLPORT)
 #include <string>                      // std::string - sigh!
#endif /* __STLSOFT_COMPILER_IS_MSVC && __STLSOFT_CF_STL_IS_STLPORT */
#include <stdexcept>                    // std::out_of_range

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_typename_param_k P
        >
class frame_array_1d;

template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_typename_param_k P
        >
class frame_array_2d;

template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_size_t N2
        ,   ss_typename_param_k P
        >
class frame_array_3d;

template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_size_t N2
        ,   ss_size_t N3
        ,   ss_typename_param_k P
        >
class frame_array_4d;

template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_size_t N2
        ,   ss_size_t N3
        ,   ss_size_t N4
        ,   ss_typename_param_k P
        >
class frame_array_5d;

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

////////////////////////////////////////////////////////////////////////////////
// Classes

// class frame_array_1d
/// 1 dimensional frame array
///
/// \param T The value type
/// \param N0 The first dimension extent
/// \param P The construction policy type
template<   ss_typename_param_k T
        ,   ss_size_t N0
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k P   =   do_construction<T>
#else
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class frame_array_1d
    : public null_allocator<T>
{
public:
    typedef frame_array_1d<T, N0, P>                        class_type;
    typedef T                                               dimension_type;
    typedef null_allocator<T>                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
//protected:
    frame_array_1d(T *data/* [N0] */);
public:
    frame_array_1d();
    frame_array_1d(value_type const &t);
    ~frame_array_1d();

// Access
public:
    reference               at(index_type i0);
    const_reference         at(index_type i0) const;

    reference               at_unchecked(index_type i0);
    const_reference         at_unchecked(index_type i0) const;

    reference               operator [](index_type i0);
    const_reference         operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    static index_type       dimension0();
    static index_type       size();
    static ss_bool_t        empty();
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const    *data() const;

// Implementation
protected:
    index_type  _calc_index(index_type i0) const;
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
protected:
    T * const           m_data;
    T                   m_data_[N0];

#if 0
    template <ss_size_t N1>
    friend class frame_array_2d<T, N0, N1, P>;
#endif /* 0 */

// Not to be implemented
private:
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_MSVC)
public:
#else
private:
#endif /* compiler */
    frame_array_1d(class_type const &rhs);
private:
    class_type const &operator =(class_type const &rhs);
};

// class frame_array_2d
/// 2 dimensional frame array
///
/// \param T The value type
/// \param N0 The first dimension extent
/// \param N1 The second dimension extent
/// \param P The construction policy type
template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k P   =   do_construction<T>
#else
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class frame_array_2d
    : public null_allocator<T>
{
public:
    typedef frame_array_2d<T, N0, N1, P>                    class_type;
    typedef frame_array_1d<T, N1, P>                        dimension_type;
    typedef null_allocator<T>                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
//protected:
//    frame_array_2d(T data[N0][N1]);
    frame_array_2d(T *data);
public:
    frame_array_2d();
    frame_array_2d(value_type const &t);
    ~frame_array_2d();

// Operations
public:
    reference               at(index_type i0, index_type i1);
    const_reference         at(index_type i0, index_type i1) const;

    reference               at_unchecked(index_type i0, index_type i1);
    const_reference         at_unchecked(index_type i0, index_type i1) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    static index_type       dimension0();
    static index_type       dimension1();
    static index_type       size();
    static ss_bool_t        empty();
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
protected:
    index_type  _calc_index(index_type i0, index_type i1) const;
    void        _range_check(index_type i0, index_type i1) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
protected:
    T * const           m_data;
    T                   m_data_[N0 * N1];

// Not to be implemented
private:
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_MSVC)
public:
#else
private:
#endif /* compiler */
    frame_array_2d(class_type const &rhs);
private:
    class_type const &operator =(class_type const &rhs);
};

// class frame_array_3d
/// 3 dimensional frame array
///
/// \param T The value type
/// \param N0 The first dimension extent
/// \param N1 The second dimension extent
/// \param N2 The third dimension extent
/// \param P The construction policy type
template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_size_t N2
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k P   =   do_construction<T>
#else
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class frame_array_3d
    : public null_allocator<T>
{
public:
    typedef frame_array_3d<T, N0, N1, N2, P>                class_type;
    typedef frame_array_2d<T, N1, N2, P>                    dimension_type;
    typedef null_allocator<T>                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
//protected:
//    frame_array_3d(T data[N0][N1][N2]);
    frame_array_3d(T *data);
public:
    frame_array_3d();
    frame_array_3d(value_type const &t);
    ~frame_array_3d();

// Operations
public:
    reference               at(index_type i0, index_type i1, index_type i2);
    const_reference         at(index_type i0, index_type i1, index_type i2) const;

    reference               at_unchecked(index_type i0, index_type i1, index_type i2);
    const_reference         at_unchecked(index_type i0, index_type i1, index_type i2) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    static index_type       dimension0();
    static index_type       dimension1();
    static index_type       dimension2();
    static index_type       size();
    static ss_bool_t        empty();
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
protected:
    index_type  _calc_index(index_type i0, index_type i1, index_type i2) const;
    void        _range_check(index_type i0, index_type i1, index_type i2) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
protected:
    T * const           m_data;
    T                   m_data_[N0 * N1 * N2];

// Not to be implemented
private:
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_MSVC)
public:
#else
private:
#endif /* compiler */
    frame_array_3d(class_type const &rhs);
private:
    class_type const &operator =(class_type const &rhs);
};


// class frame_array_4d
/// 4 dimensional frame array
///
/// \param T The value type
/// \param N0 The first dimension extent
/// \param N1 The second dimension extent
/// \param N2 The third dimension extent
/// \param P The construction policy type
template<   ss_typename_param_k T
        ,   ss_size_t N0
        ,   ss_size_t N1
        ,   ss_size_t N2
        ,   ss_size_t N3
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k P   =   do_construction<T>
#else
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class frame_array_4d
    : public null_allocator<T>
{
public:
    typedef frame_array_4d<T, N0, N1, N2, N3, P>            class_type;
    typedef frame_array_3d<T, N1, N2, N3, P>                dimension_type;
    typedef null_allocator<T>                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
//protected:
//    frame_array_4d(T data[N0][N1][N2][N3]);
    frame_array_4d(T *data);
public:
    frame_array_4d();
    frame_array_4d(value_type const &t);
    ~frame_array_4d();

// Operations
public:
    reference               at(index_type i0, index_type i1, index_type i2, index_type i3);
    const_reference         at(index_type i0, index_type i1, index_type i2, index_type i3) const;

    reference               at_unchecked(index_type i0, index_type i1, index_type i2, index_type i3);
    const_reference         at_unchecked(index_type i0, index_type i1, index_type i2, index_type i3) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    static index_type       dimension0();
    static index_type       dimension1();
    static index_type       dimension2();
    static index_type       dimension3();
    static index_type       size();
    static ss_bool_t        empty();
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
protected:
    index_type  _calc_index(index_type i0, index_type i1, index_type i2, index_type i3) const;
    void        _range_check(index_type i0, index_type i1, index_type i2, index_type i3) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0, index_type i1, index_type i2) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
protected:
    T * const           m_data;
    T                   m_data_[N0 * N1 * N2 * N3];

// Not to be implemented
private:
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_MSVC)
public:
#else
private:
#endif /* compiler */
    frame_array_4d(class_type const &rhs);
private:
    class_type const &operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

// frame_array_1d

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::index_type frame_array_1d<T, N0, P>::_calc_index(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0) const
{
    return i0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline void frame_array_1d<T, N0, P>::_range_check(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < N0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline frame_array_1d<T, N0, P>::frame_array_1d(T *data/* [N0] */)
    : m_data(data)
{}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline frame_array_1d<T, N0, P>::frame_array_1d()
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            allocator_type::construct(m_data + _calc_index(i0));
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline frame_array_1d<T, N0, P>::frame_array_1d(value_type const &t)
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            allocator_type::construct(m_data + _calc_index(i0), t);
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline frame_array_1d<T, N0, P>::~frame_array_1d()
{
    if(policy_type::value)
    {
        if(m_data == m_data_)
        {
            for(index_type i0 = 0; i0 < N0; ++i0)
            {
                allocator_type::destroy(m_data + _calc_index(i0));
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type &frame_array_1d<T, N0, P>::at(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0)
{
    _range_check(i0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type const &frame_array_1d<T, N0, P>::at(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0) const
{
    _range_check(i0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type &frame_array_1d<T, N0, P>::at_unchecked(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0)
{
    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type const &frame_array_1d<T, N0, P>::at_unchecked(ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0) const
{
    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type &frame_array_1d<T, N0, P>::operator [](ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0)
{
    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type const &frame_array_1d<T, N0, P>::operator [](ss_typename_type_k frame_array_1d<T, N0, P>::index_type i0) const
{
    return m_data[i0];
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::reference frame_array_1d<T, N0, P>::front()
{
    return at(0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::reference frame_array_1d<T, N0, P>::back()
{
    return at(N0 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_reference frame_array_1d<T, N0, P>::front() const
{
    return at(0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_reference frame_array_1d<T, N0, P>::back() const
{
    return at(N0 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_1d<T, N0, P>::index_type frame_array_1d<T, N0, P>::dimension0()
{
    return N0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_1d<T, N0, P>::index_type frame_array_1d<T, N0, P>::size()
{
    return N0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline /* static */ ss_bool_t frame_array_1d<T, N0, P>::empty()
{
    return false;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_1d<T, N0, P>::index_type frame_array_1d<T, N0, P>::max_size()
{
    return size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::iterator frame_array_1d<T, N0, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::iterator frame_array_1d<T, N0, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_iterator frame_array_1d<T, N0, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_iterator frame_array_1d<T, N0, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::reverse_iterator frame_array_1d<T, N0, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::reverse_iterator frame_array_1d<T, N0, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_reverse_iterator frame_array_1d<T, N0, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::const_reverse_iterator frame_array_1d<T, N0, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_typename_param_k P>
inline ss_typename_type_k frame_array_1d<T, N0, P>::value_type const *frame_array_1d<T, N0, P>::data() const
{
    return m_data;
}


// frame_array_2d

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type frame_array_2d<T, N0, N1, P>::_calc_index(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1) const
{
    return (i0 * N1) + i1;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline void frame_array_2d<T, N0, N1, P>::_range_check(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0); ((void)i1); ((void)N1);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < N0) ||
        !(i1 < N1))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0 && i1 < N1);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline void frame_array_2d<T, N0, N1, P>::_range_check(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < N0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline frame_array_2d<T, N0, N1, P>::frame_array_2d(T *data)
    : m_data(data)
{}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline frame_array_2d<T, N0, N1, P>::frame_array_2d()
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                allocator_type::construct(m_data + _calc_index(i0, i1));
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline frame_array_2d<T, N0, N1, P>::frame_array_2d(value_type const &t)
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                allocator_type::construct(m_data + _calc_index(i0, i1), t);
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline frame_array_2d<T, N0, N1, P>::~frame_array_2d()
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                allocator_type::destroy(m_data + _calc_index(i0, i1));
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reference frame_array_2d<T, N0, N1, P>::at(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1)
{
    _range_check(i0, i1);

    return *(m_data + i0 * N1 + i1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reference frame_array_2d<T, N0, N1, P>::at(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1) const
{
    _range_check(i0, i1);

    return *(m_data + i0 * N1 + i1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reference frame_array_2d<T, N0, N1, P>::at_unchecked(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1)
{
    return *(m_data + i0 * N1 + i1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reference frame_array_2d<T, N0, N1, P>::at_unchecked(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0, ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i1) const
{
    return *(m_data + i0 * N1 + i1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type frame_array_2d<T, N0, N1, P>::at(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type const frame_array_2d<T, N0, N1, P>::at(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type frame_array_2d<T, N0, N1, P>::at_unchecked(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0)
{
    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type const frame_array_2d<T, N0, N1, P>::at_unchecked(ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0) const
{
    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type frame_array_2d<T, N0, N1, P>::operator [](ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0)
{
    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_2d<T, N0, N1, P>::dimension_type frame_array_2d<T, N0, N1, P>::operator [](ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type i0) const
{
    return dimension_type(m_data + i0 * N1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reference frame_array_2d<T, N0, N1, P>::front()
{
    return at(0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reference frame_array_2d<T, N0, N1, P>::back()
{
    return at(N0 - 1, N1 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reference frame_array_2d<T, N0, N1, P>::front() const
{
    return at(0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reference frame_array_2d<T, N0, N1, P>::back() const
{
    return at(N0 - 1, N1 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type frame_array_2d<T, N0, N1, P>::dimension0()
{
    return N0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type frame_array_2d<T, N0, N1, P>::dimension1()
{
    return N1;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type frame_array_2d<T, N0, N1, P>::size()
{
    return N0 * N1;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline /* static */ ss_bool_t frame_array_2d<T, N0, N1, P>::empty()
{
    return false;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_2d<T, N0, N1, P>::index_type frame_array_2d<T, N0, N1, P>::max_size()
{
    return size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::iterator frame_array_2d<T, N0, N1, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::iterator frame_array_2d<T, N0, N1, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_iterator frame_array_2d<T, N0, N1, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_iterator frame_array_2d<T, N0, N1, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reverse_iterator frame_array_2d<T, N0, N1, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::reverse_iterator frame_array_2d<T, N0, N1, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reverse_iterator frame_array_2d<T, N0, N1, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::const_reverse_iterator frame_array_2d<T, N0, N1, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_typename_param_k P>
inline ss_typename_type_k frame_array_2d<T, N0, N1, P>::value_type const *frame_array_2d<T, N0, N1, P>::data() const
{
    return m_data;
}

// frame_array_3d

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::_calc_index(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2) const
{
    return ((i0 * N1) + i1) * N2 + i2;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline void frame_array_3d<T, N0, N1, N2, P>::_range_check(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0); ((void)i1); ((void)N1); ((void)i2); ((void)N2);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < N0) ||
        !(i1 < N1) ||
        !(i2 < N2))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0 && i1 < N1 && i2 < N2);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline void frame_array_3d<T, N0, N1, N2, P>::_range_check(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < N0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline frame_array_3d<T, N0, N1, N2, P>::frame_array_3d(T *data)
    : m_data(data)
{}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline frame_array_3d<T, N0, N1, N2, P>::frame_array_3d()
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    allocator_type::construct(m_data + _calc_index(i0, i1, i2));
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline frame_array_3d<T, N0, N1, N2, P>::frame_array_3d(value_type const &t)
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    allocator_type::construct(m_data + _calc_index(i0, i1, i2));
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline frame_array_3d<T, N0, N1, N2, P>::~frame_array_3d()
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    allocator_type::destroy(m_data + _calc_index(i0, i1, i2));
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::value_type &frame_array_3d<T, N0, N1, N2, P>::at(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2)
{
    _range_check(i0, i1, i2);

    return *(m_data + i0 * N1 + i1 * N2 + i2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::value_type const &frame_array_3d<T, N0, N1, N2, P>::at(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2) const
{
    _range_check(i0, i1, i2);

    return *(m_data + i0 * N1 + i1 * N2 + i2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::value_type &frame_array_3d<T, N0, N1, N2, P>::at_unchecked(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2)
{
    return *(m_data + i0 * N1 + i1 * N2 + i2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::value_type const &frame_array_3d<T, N0, N1, N2, P>::at_unchecked(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i1, ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i2) const
{
    return *(m_data + i0 * N1 + i1 * N2 + i2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::at(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::at(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::at_unchecked(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0)
{
    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::at_unchecked(ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0) const
{
    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::operator [](ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0)
{
    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::dimension_type frame_array_3d<T, N0, N1, N2, P>::operator [](ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type i0) const
{
    return dimension_type(m_data + i0 * N1 * N2);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::reference frame_array_3d<T, N0, N1, N2, P>::front()
{
    return at(0, 0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::reference frame_array_3d<T, N0, N1, N2, P>::back()
{
    return at(N0 - 1, N1 - 1, N2 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_reference frame_array_3d<T, N0, N1, N2, P>::front() const
{
    return at(0, 0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_reference frame_array_3d<T, N0, N1, N2, P>::back() const
{
    return at(N0 - 1, N1 - 1, N2 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::dimension0()
{
    return N0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::dimension1()
{
    return N1;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::dimension2()
{
    return N2;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::size()
{
    return N0 * N1 * N2;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_bool_t frame_array_3d<T, N0, N1, N2, P>::empty()
{
    return false;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::index_type frame_array_3d<T, N0, N1, N2, P>::max_size()
{
    return size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::iterator frame_array_3d<T, N0, N1, N2, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::iterator frame_array_3d<T, N0, N1, N2, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_iterator frame_array_3d<T, N0, N1, N2, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_iterator frame_array_3d<T, N0, N1, N2, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::reverse_iterator frame_array_3d<T, N0, N1, N2, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::reverse_iterator frame_array_3d<T, N0, N1, N2, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_reverse_iterator frame_array_3d<T, N0, N1, N2, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::const_reverse_iterator frame_array_3d<T, N0, N1, N2, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_typename_param_k P>
inline ss_typename_type_k frame_array_3d<T, N0, N1, N2, P>::value_type const *frame_array_3d<T, N0, N1, N2, P>::data() const
{
    return m_data;
}

// frame_array_4d

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::_calc_index(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3) const
{
    return (((i0 * N1) + i1) * N2 + i2) * N3 + i3;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline void frame_array_4d<T, N0, N1, N2, N3, P>::_range_check(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)i1); ((void)i2); ((void)i3);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < N0) ||
        !(i1 < N1) ||
        !(i2 < N2) ||
        !(i3 < N3))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0 && i1 < N1 && i2 < N2 && i3 < N3);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline void frame_array_4d<T, N0, N1, N2, N3, P>::_range_check(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
    ((void)i0); ((void)N0);

#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < N0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("frame array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("frame array index out of range", i0 < N0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline frame_array_4d<T, N0, N1, N2, N3, P>::frame_array_4d()
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    for(index_type i3 = 0; i3 < N3; ++i3)
                    {
                        allocator_type::construct(m_data + _calc_index(i0, i1, i2, i3));
                    }
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline frame_array_4d<T, N0, N1, N2, N3, P>::frame_array_4d(value_type const &t)
    : m_data(m_data_)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    for(index_type i3 = 0; i3 < N3; ++i3)
                    {
                        allocator_type::construct(m_data + _calc_index(i0, i1, i2, i3), t);
                    }
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline frame_array_4d<T, N0, N1, N2, N3, P>::~frame_array_4d()
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < N0; ++i0)
        {
            for(index_type i1 = 0; i1 < N1; ++i1)
            {
                for(index_type i2 = 0; i2 < N2; ++i2)
                {
                    for(index_type i3 = 0; i3 < N3; ++i3)
                    {
                        allocator_type::destroy(m_data + _calc_index(i0, i1, i2, i3));
                    }
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::value_type &frame_array_4d<T, N0, N1, N2, N3, P>::at(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3)
{
    _range_check(i0, i1, i2, i3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::value_type const &frame_array_4d<T, N0, N1, N2, N3, P>::at(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3) const
{
    _range_check(i0, i1, i2, i3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::value_type &frame_array_4d<T, N0, N1, N2, N3, P>::at_unchecked(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3)
{
    stlsoft_message_assert("frame array index out of range", i0 < N0 && i1 < N1 && i2 < N2 && i3 < N3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::value_type const &frame_array_4d<T, N0, N1, N2, N3, P>::at_unchecked(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i1, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i2, ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i3) const
{
    stlsoft_message_assert("frame array index out of range", i0 < N0 && i1 < N1 && i2 < N2 && i3 < N3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::at(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::at(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::at_unchecked(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0)
{
    stlsoft_message_assert("frame array index out of range", i0 < N0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::at_unchecked(ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0) const
{
    stlsoft_message_assert("frame array index out of range", i0 < N0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::operator [](ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0)
{
    stlsoft_message_assert("frame array index out of range", i0 < N0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline const ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::dimension_type frame_array_4d<T, N0, N1, N2, N3, P>::operator [](ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type i0) const
{
    stlsoft_message_assert("frame array index out of range", i0 < N0);

    return dimension_type(m_data + i0 * N1 * N2 * N3);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::reference frame_array_4d<T, N0, N1, N2, N3, P>::front()
{
    return at(0, 0, 0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::reference frame_array_4d<T, N0, N1, N2, N3, P>::back()
{
    return at(N0 - 1, N1 - 1, N2 - 1, N3 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_reference frame_array_4d<T, N0, N1, N2, N3, P>::front() const
{
    return at(0, 0, 0, 0);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_reference frame_array_4d<T, N0, N1, N2, N3, P>::back() const
{
    return at(N0 - 1, N1 - 1, N2 - 1, N3 - 1);
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::dimension0()
{
    return N0;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::dimension1()
{
    return N1;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::dimension2()
{
    return N2;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::dimension3()
{
    return N3;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::size()
{
    return N0 * N1 * N2 * N3;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_bool_t frame_array_4d<T, N0, N1, N2, N3, P>::empty()
{
    return false;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline /* static */ ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::index_type frame_array_4d<T, N0, N1, N2, N3, P>::max_size()
{
    return size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::iterator frame_array_4d<T, N0, N1, N2, N3, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::iterator frame_array_4d<T, N0, N1, N2, N3, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_iterator frame_array_4d<T, N0, N1, N2, N3, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_iterator frame_array_4d<T, N0, N1, N2, N3, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::reverse_iterator frame_array_4d<T, N0, N1, N2, N3, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::reverse_iterator frame_array_4d<T, N0, N1, N2, N3, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_reverse_iterator frame_array_4d<T, N0, N1, N2, N3, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::const_reverse_iterator frame_array_4d<T, N0, N1, N2, N3, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_size_t N0, ss_size_t N1, ss_size_t N2, ss_size_t N3, ss_typename_param_k P>
inline ss_typename_type_k frame_array_4d<T, N0, N1, N2, N3, P>::value_type const *frame_array_4d<T, N0, N1, N2, N3, P>::data() const
{
    return m_data;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_FRAME_ARRAY */

/* ////////////////////////////////////////////////////////////////////////// */
