/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_sign_traits.h
 *
 * Purpose:     sign_traits classes.
 *
 * Created:     16th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_SIGN_TRAITS
#define _STLSOFT_INCL_H_STLSOFT_SIGN_TRAITS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_SIGN_TRAITS_MAJOR        1
#define _STLSOFT_VER_H_STLSOFT_SIGN_TRAITS_MINOR        1
#define _STLSOFT_VER_H_STLSOFT_SIGN_TRAITS_REVISION     2
#define _STLSOFT_VER_H_STLSOFT_SIGN_TRAITS_EDIT         17
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// struct sign_traits

#ifdef __STLSOFT_DOCUMENTATION_SKIP_SECTION
/// Traits for determining the signed, unsigned and alternate-signed type
///
/// sign_traits is a traits class for acquiring the corresponding signed,
/// unsigned, and alternate-signed type eg.
///
/// &nbsp;&nbsp;<code>sign_traits<ss_sint16_t>::signed_type == ss_sint16_t</code><br>
/// &nbsp;&nbsp;<code>sign_traits<ss_sint16_t>::unsigned_type == ss_uint16_t</code><br>
/// &nbsp;&nbsp;<code>sign_traits<ss_sint16_t>::alt_sign_type == ss_uint16_t</code><br>
///
/// \param T The char type
template <ss_typename_param_k T>
struct sign_traits
{
    enum
    {
        bits = 8                        //!< The type size, in bits
    };

    typedef ss_char_a_t signed_type;    //!< The signed type
    typedef ss_uint8_t  unsigned_type;  //!< The unsigned type
    typedef ss_uint8_t  alt_sign_type;  //!< The alternate-signed type
};

#else

template <ss_typename_param_k T>
struct sign_traits;

/* char */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_char_a_t>
{
    enum { bits = 8 };

    typedef ss_char_a_t signed_type;
    typedef ss_uint8_t  unsigned_type;
    typedef ss_uint8_t  alt_sign_type;
};

/* s/uint8 */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_sint8_t>
{
    enum { bits = 8 };

    typedef ss_sint8_t  signed_type;
    typedef ss_uint8_t  unsigned_type;
    typedef ss_uint8_t  alt_sign_type;
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_uint8_t>
{
    enum { bits = 8 };

    typedef ss_uint8_t  signed_type;
    typedef ss_sint8_t  unsigned_type;
    typedef ss_sint8_t  alt_sign_type;
};

/* s/uint16 */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_sint16_t>
{
    enum { bits = 16 };

    typedef ss_sint16_t signed_type;
    typedef ss_uint16_t unsigned_type;
    typedef ss_uint16_t alt_sign_type;
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_uint16_t>
{
    enum { bits = 16 };

    typedef ss_uint16_t signed_type;
    typedef ss_sint16_t unsigned_type;
    typedef ss_sint16_t alt_sign_type;
};

/* s/uint32 */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_sint32_t>
{
    enum { bits = 32 };

    typedef ss_sint32_t signed_type;
    typedef ss_uint32_t unsigned_type;
    typedef ss_uint32_t alt_sign_type;
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_uint32_t>
{
    enum { bits = 32 };

    typedef ss_uint32_t signed_type;
    typedef ss_sint32_t unsigned_type;
    typedef ss_sint32_t alt_sign_type;
};

#ifdef __STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT
/* s/uint64 */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_sint64_t>
{
    enum { bits = 64 };

    typedef ss_sint64_t signed_type;
    typedef ss_uint64_t unsigned_type;
    typedef ss_uint64_t alt_sign_type;
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct sign_traits<ss_uint64_t>
{
    enum { bits = 64 };

    typedef ss_uint64_t signed_type;
    typedef ss_sint64_t unsigned_type;
    typedef ss_sint64_t alt_sign_type;
};
#endif /* !__STLSOFT_CF_NATIVE_64BIT_INTEGER_SUPPORT */

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_SIGN_TRAITS */

/* ////////////////////////////////////////////////////////////////////////// */
