/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_simple_string.h (formerly MSString.h, ::SynesisDev)
 *
 * Purpose:     simple_string class.
 *
 * Created:     19th March 1993
 * Updated:     17th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_SIMPLE_STRING
#define _STLSOFT_INCL_H_STLSOFT_SIMPLE_STRING

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_STRING_MAJOR     1
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_STRING_MINOR     9
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_STRING_REVISION  2
#define _STLSOFT_VER_H_STLSOFT_SIMPLE_STRING_EDIT      113
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"                   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_NEW_ALLOCATOR
 #include "stlsoft_new_allocator.h"     // stlsoft::new_allocator
#endif /* !_STLSOFT_INCL_H_STLSOFT_NEW_ALLOCATOR */
#if !defined(_STLSOFT_NO_NAMESPACE) && \
    !defined(_STLSOFT_SIMPLE_STRING_NO_USE_STLSOFT_CHAR_TRAITS)
 #ifndef _STLSOFT_INCL_H_STLSOFT_CHAR_TRAITS
  #include "stlsoft_char_traits.h"      // stlsoft::char_traits
 #endif /* !_STLSOFT_INCL_H_STLSOFT_CHAR_TRAITS */
#else
 #include <string>                      // std::char_traits
#endif /* !_STLSOFT_NO_NAMESPACE && !_STLSOFT_SIMPLE_STRING_NO_USE_STLSOFT_CHAR_TRAITS */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"          // stlsoft iterator bases
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"       // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef _STLSOFT_INCL_H_STLSOFT_SIMPLE_ALGORITHMS
 #include "stlsoft_simple_algorithms.h" // stlsoft::for_each_set_value
#endif /* !_STLSOFT_INCL_H_STLSOFT_SIMPLE_ALGORITHMS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_string String Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating strings

/// \weakgroup stlsoft_string_library String Library (STLSoft)
/// \ingroup STLSoft libraries_string
/// \brief This library provides facilities for defining and manipulating strings
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class basic_simple_string
/// Simple string class
///
/// \param C The character type
/// \param T The traits type. On translators that support default template arguments this is defaulted to char_traits<C>
/// \param A The allocator type. On translators that support default template arguments this is defaulted to new_allocator<C>
template<   ss_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
 #if !defined(_STLSOFT_NO_NAMESPACE) && \
     !defined(_STLSOFT_SIMPLE_STRING_NO_USE_STLSOFT_CHAR_TRAITS)
        ,   ss_typename_param_k T = stlsoft_ns_qual(char_traits)<C>
 #else
        ,   ss_typename_param_k T = stlsoft_ns_qual_std(char_traits)<C>
 #endif /* !_STLSOFT_NO_NAMESPACE && !_STLSOFT_SIMPLE_STRING_NO_USE_STLSOFT_CHAR_TRAITS */
        ,   ss_typename_param_k A = new_allocator<C>
#else
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_simple_string
{
public:
    /// The value type
    typedef C                               value_type;
    /// The traits type
    typedef T                               traits_type;
    /// The allocator type
    typedef A                               allocator_type;
    /// The current parameterisation of the type
    typedef basic_simple_string<C, T, A>    class_type;
    /// The character type
    typedef value_type                      char_type;
    /// The pointer type
    typedef value_type                      *pointer;
    /// The non-mutable (const) pointer type
    typedef value_type const                *const_pointer;
    /// The reference type
    typedef value_type                      &reference;
    /// The non-mutable (const) reference type
    typedef value_type const                &const_reference;
    /// The size type
    typedef ss_size_t                       size_type;
    /// The difference type
    typedef ss_ptrdiff_t                    difference_type;

    /// The iterator type
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    /// The non-mutating (const) iterator type
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// The reverse iterator type
    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    /// The non-mutating (const) reverse iterator type
    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

/// \name Construction
//@{
public:
    /// Default constructor
    basic_simple_string();
    /// Copy constructor
    basic_simple_string(class_type const &rhs);
    /// Construct from the given string at the specified position
    basic_simple_string(class_type const &s, size_type pos);
    /// Construct with \c n characters from the given string at the specified position
    basic_simple_string(class_type const &s, size_type pos, size_type n);
    /// Construct from the given character string
    basic_simple_string(char_type const *s); // Not, not explicit. Sigh
    /// Construct with \c n characters from the given character string
    basic_simple_string(char_type const *s, size_type n);
    /// Construct with \c n characters each set to \c ch
    basic_simple_string(size_type n, char_type ch);
    /// Construct from the range [first:last)
#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
    basic_simple_string(char_type const *first, char_type const *last);
#else
    template <ss_typename_param_k II>
    basic_simple_string(II first, II last)
        : m_buffer(0)
    {
        assign(first, last);
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */
    /// Destructor
    ~basic_simple_string();
//@}

/// \name Assignment
//@{
public:
    /// Assigns from the given character string
    class_type &assign(const char_type *s);
    /// Assigns with \c n characters from the given character string
    class_type &assign(const char_type *s, size_type n);
    /// Assigns with \c n characters from the given character string at the specified position
    class_type &assign(class_type const &str, size_type pos, size_type n);
    /// Assigns from the given string
    class_type &assign(class_type const &str);
    /// Assigns \c n characters with the value \c ch
    class_type &assign(size_type n, char_type ch);
    /// Assigns from the range [first:last)
#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
    class_type &assign(const_iterator first, const_iterator last);
#else
    template <ss_typename_param_k II>
    class_type &assign(II first, II last)
    {
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC)
        typedef ss_typename_type_k std::iterator_traits<II> traits_t;

        return _assign(first, last, traits_t::iterator_category());
#else
        return _assign(first, last, stlsoft_iterator_query_category(II, first));
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

    /// Copy assignment operator
    class_type const &operator =(class_type const &rhs);
    /// Assignment operator
    class_type const &operator =(char_type const *s);
    /// Assignment operator
    class_type const &operator =(char_type ch);
//@}

/// \name Appending
//@{
public:
    /// Appends the given character string
    class_type &append(const char_type *s);
    /// Appends \c n characters from the given character string
    class_type &append(const char_type *s, size_type n);
    /// Assigns \c n characters from the given character string at the specified position
    class_type &append(class_type const &str, size_type pos, size_type n);
    /// Appends the given string
    class_type &append(class_type const &str);
    /// Appends \c n characters with the value \c ch
    class_type &append(size_type n, char_type ch);
    /// Appends the range [first:last)
#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
    class_type &append(const_iterator first, const_iterator last);
#else
    template <ss_typename_param_k II>
    class_type &append(II first, II last)
    {
#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    defined(__STLSOFT_COMPILER_IS_GCC)
        typedef ss_typename_type_k std::iterator_traits<II> traits_t;

        return _append(first, last, traits_t::iterator_category());
#else
        return _append(first, last, stlsoft_iterator_query_category(II, first));
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

    /// Concatenation operator
    class_type &operator +=(char_type ch);
    /// Concatenation operator
    class_type &operator +=(const char_type *s);
    /// Concatenation operator
    class_type &operator +=(class_type const &rhs);

    /// Appends a single character
    void push_back(char_type ch);
//@}

/// \name Operations
//@{
public:
    /// Reserves at least n characters
    void reserve(size_type n);
    /// Swaps the contents between \c this and \c other
    void swap(class_type &other);

    /// Empties the string
    void clear();
//@}

/// \name Attributes
//@{
public:
    /// The number of elements in the string
    size_type size() const;
    /// The maximum number of elements that can be stored in the string
    size_type max_size() const;
    /// The number of elements in the string
    size_type length() const;
    /// The storage currently allocated by the string
    size_type capacity() const;
    /// Indicates whether the string is empty
    ss_bool_t empty() const;
//@}

/// \name Comparison
//@{
public:
    /// Compares \c this with the given string
    ss_sint_t compare(size_type pos, size_type n, value_type const *s, size_type cchRhs) const;
    /// Compares \c this with the given string
    ss_sint_t compare(size_type pos, size_type n, value_type const *s) const;
    /// Compares \c this with the given string
    ss_sint_t compare(value_type const *s) const;
    /// Compares \c this with the given string
    ss_sint_t compare(size_type pos, size_type n, class_type const &rhs, size_type posRhs, size_type cchRhs) const;
    /// Compares \c this with the given string
    ss_sint_t compare(size_type pos, size_type n, class_type const &rhs) const;
    /// Compares \c this with the given string
    ss_sint_t compare(class_type const &rhs) const;
//@}

/// \name Accessors
//@{
public:
    /// Returns mutable reference at the given index
    reference       operator [](size_type index);
    /// Returns non-mutable (const) reference at the given index
    const_reference operator [](size_type index) const;

    /// Returns null-terminated non-mutable (const) pointer to string data
    value_type const    *c_str() const;
    /// Returns non-mutable (const) pointer to string data
    value_type const    *data() const;

    /// Copies elements into the given destination
    size_type copy(value_type *dest, size_type cch, size_type pos = 0) const;
//@}

/// \name Iteration
//@{
public:
    /// Begins the iteration
    ///
    /// \return A non-mutable (const) iterator representing the start of the sequence
    const_iterator          begin() const;
    /// Ends the iteration
    ///
    /// \return A non-mutable (const) iterator representing the end of the sequence
    const_iterator          end() const;
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    iterator                begin();
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    iterator                end();

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// Begins the reverse iteration
    ///
    /// \return A non-mutable (const) iterator representing the start of the reverse sequence
    const_reverse_iterator  rbegin() const;
    /// Ends the reverse iteration
    ///
    /// \return A non-mutable (const) iterator representing the end of the reverse sequence
    const_reverse_iterator  rend() const;
    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    reverse_iterator        rbegin();
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    reverse_iterator        rend();
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */
//@}

/// \name Implementation
//@{
private:
    struct string_buffer
    {
        size_type   capacity;
        size_type   length;
        char_type   contents[1];
    };

    // Conversion between buffer and pointer
    static string_buffer        *_buffer_from_ptr(char_type *s);
    static string_buffer const  *_buffer_from_ptr(char_type const *s);
#if 0
    static string_buffer        *_buffer_from_ptr_null(char_type *s);
    static string_buffer const  *_buffer_from_ptr_null(char_type const *s);
#endif /* 0 */
    static char_type            *_ptr_from_buffer(string_buffer *buffer);
    static char_type const      *_ptr_from_buffer(string_buffer const *buffer);

    // Copying a buffer
    static char_type            *_copy_buffer(string_buffer *);
    static char_type            *_copy_buffer(char_type *);

    // Destroying buffer
    static void                 _destroy_buffer(string_buffer *);
    static void                 _destroy_buffer(char_type *);

    // Creating buffer
    static char_type            *_alloc_buffer(char_type const *s, size_type cch);
    static char_type            *_alloc_buffer(char_type const *s);
    static char_type            *_realloc_buffer(string_buffer *buffer, size_type cch);
    static char_type            *_realloc_buffer(string_buffer *buffer, char_type const *s, size_type cch);
    static char_type            *_realloc_buffer(string_buffer *buffer, char_type const *s);

    // Iteration
    pointer _begin();
    pointer _end();

    // Invariance
    ss_bool_t _is_valid() const;

    // Empty string
    static char_type *_empty_string();

    // Comparison
    static ss_sint_t _compare(char_type const *lhs, size_type lhs_len, char_type const *rhs, size_type rhs_len);

    // Assignment
#if defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
    template <ss_typename_param_k II>
    class_type &_assign(II first, II last, stlsoft_ns_qual_std(input_iterator_tag))
    {
        stlsoft_ns_qual_std(copy)(first, last, stlsoft_ns_qual_std(back_inserter)(*this));

        stlsoft_assert(_is_valid());
        return *this;
    }
    template <ss_typename_param_k II>
    class_type &_assign(II first, II last, stlsoft_ns_qual_std(forward_iterator_tag))
    {
        typedef auto_buffer<char_type, allocator_type>  buffer_t;

        buffer_t    buffer(stlsoft_ns_qual_std(distance)(first, last));

        stlsoft_ns_qual_std(copy)(first, last, buffer.begin());
        assign(buffer, buffer.size());

        stlsoft_assert(_is_valid());
        return *this;
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

    // Appending
#if defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
    template <ss_typename_param_k II>
    class_type &_append(II first, II last, stlsoft_ns_qual_std(input_iterator_tag))
    {
        stlsoft_ns_qual_std(copy)(first, last, stlsoft_ns_qual_std(back_inserter)(*this));

        stlsoft_assert(_is_valid());
        return *this;
    }
    template <ss_typename_param_k II>
    class_type &_append(II first, II last, stlsoft_ns_qual_std(forward_iterator_tag))
    {
        typedef auto_buffer<char_type, allocator_type>  buffer_t;

        buffer_t    buffer(stlsoft_ns_qual_std(distance)(first, last));

        stlsoft_ns_qual_std(copy)(first, last, buffer.begin());
        append(buffer, buffer.size());

        stlsoft_assert(_is_valid());
        return *this;
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */
//@}

/// \name Members
//@{
private:
    char_type   *m_buffer;
//@}
};

/* /////////////////////////////////////////////////////////////////////////////
 * Operators
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

// operator ==

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator ==(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) == 0;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator ==(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator ==(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) == 0;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator ==(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator ==(C *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) == 0;
}

// operator !=

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator !=(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) != 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator !=(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator !=(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) != 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator !=(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator !=(C const *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) != 0;
}

// operator <

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator <(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) < 0;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator <(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) < 0;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator <(C const *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) > 0;
}

// operator <=

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator <=(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) <= 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <=(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator <=(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) <= 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <=(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator <=(C const *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) >= 0;
}

// operator >

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator >(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) > 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator >(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) > 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator >(C const *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) < 0;
}

// operator >=

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t operator >=(basic_simple_string<C, T, A> const &lhs, basic_simple_string<C, T, A> const &rhs)
{
    return lhs.compare(rhs) >= 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >=(basic_simple_string<C, T, A> const &lhs, ss_typename_type_k basic_simple_string<C, T, A>::char_type const *rhs)
#else
inline ss_bool_t operator >=(basic_simple_string<C, T, A> const &lhs, C const *rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.compare(rhs) >= 0;
}
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >=(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *lhs, basic_simple_string<C, T, A> const &rhs)
#else
inline ss_bool_t operator >=(C const *lhs, basic_simple_string<C, T, A> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return rhs.compare(lhs) <= 0;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

// Implementation

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_alloc_buffer(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *s, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    size_type   length  =   traits_type::length_max_null(s, cch);
    size_type   size    =   cch;

    if(cch < length)
    {
        length = cch;
    }

    size += 1;                  // For null terminator
    size = (0xf + size) & ~0xf; // Round up to 16

    ss_typename_type_k allocator_type::rebind<ss_byte_t>::other    allocator;

    ss_byte_t       *raw_buffer =   allocator.allocate(size * sizeof(char_type) + stlsoft_raw_offsetof(string_buffer, contents));
    string_buffer   *buffer     =   static_cast<string_buffer*>(static_cast<void*>(raw_buffer));

    if(buffer != 0)
    {
        if(s == 0)
        {
            buffer->contents[0] = traits_type::to_char_type(0);
        }
        else
        {
            traits_type::copy(buffer->contents, s, length);
            buffer->contents[length] = 0;
        }

        buffer->length      =   length;
        buffer->capacity    =   size;
    }

    return _ptr_from_buffer(buffer);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_alloc_buffer(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *s)
{
    return (s == 0) ? 0 : _alloc_buffer(s, traits_type::length(s));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_copy_buffer(ss_typename_type_k basic_simple_string<C, T, A>::string_buffer *buffer)
{
    return _alloc_buffer(buffer->contents, buffer->length);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_copy_buffer(ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    return (s == 0) ? 0 : _copy_buffer(_buffer_from_ptr(s));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ void basic_simple_string<C, T, A>::_destroy_buffer(ss_typename_type_k basic_simple_string<C, T, A>::string_buffer *buffer)
{
    ss_typename_type_k allocator_type::rebind<ss_byte_t>::other allocator;

    allocator.deallocate(static_cast<ss_byte_t*>(static_cast<void*>(buffer)), 0);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ void basic_simple_string<C, T, A>::_destroy_buffer(ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    _destroy_buffer(_buffer_from_ptr(s));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::string_buffer *basic_simple_string<C, T, A>::_buffer_from_ptr(ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    stlsoft_message_assert("Attempt to convert a null string_buffer in basic_simple_string", s != 0);

    return reinterpret_cast<string_buffer*>(const_cast<void*>(ptr_byte_offset(s, -static_cast<ss_ptrdiff_t>(stlsoft_raw_offsetof(string_buffer, contents)))));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::string_buffer const *basic_simple_string<C, T, A>::_buffer_from_ptr(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *s)
{
    stlsoft_message_assert("Attempt to convert a null string_buffer in basic_simple_string", s != 0);

    return reinterpret_cast<string_buffer const*>(ptr_byte_offset(s, -static_cast<ss_ptrdiff_t>(stlsoft_raw_offsetof(string_buffer, contents))));
}

#if 0
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::string_buffer *basic_simple_string<C, T, A>::_buffer_from_ptr_null(ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    return (s == 0) ? 0 : _buffer_from_ptr(s);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::string_buffer const *basic_simple_string<C, T, A>::_buffer_from_ptr_null(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *s)
{
    return (s == 0) ? 0 : _buffer_from_ptr(s);
}
#endif /* 0 */

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_ptr_from_buffer(ss_typename_type_k basic_simple_string<C, T, A>::string_buffer *buffer)
{
    return (buffer == 0) ? 0 : buffer->contents;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type const *basic_simple_string<C, T, A>::_ptr_from_buffer(ss_typename_type_k basic_simple_string<C, T, A>::string_buffer const *buffer)
{
    return (buffer == 0) ? 0 : buffer->contents;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::pointer basic_simple_string<C, T, A>::_begin()
{
    return m_buffer;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::pointer basic_simple_string<C, T, A>::_end()
{
    return m_buffer + length();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t basic_simple_string<C, T, A>::_is_valid() const
{
    if(m_buffer != 0)
    {
        string_buffer const *buffer =   _buffer_from_ptr(const_cast<char_type const*>(m_buffer));

        if(buffer->capacity < 1)
        {
            return false;
        }
        else if(buffer->capacity < buffer->length)
        {
            return false;
        }
        else
        {
            size_type   len =   traits_type::length(buffer->contents);

            if(buffer->length != len)
            {
                return false;
            }
        }
    }

    return true;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_typename_type_k basic_simple_string<C, T, A>::char_type *basic_simple_string<C, T, A>::_empty_string()
{
    static char_type    s_es[1] = 
    {
#if !defined(__STLSOFT_COMPILER_IS_DMC)
        traits_type::to_char_type(0)
#else
        0
#endif /* !__STLSOFT_COMPILER_IS_DMC */
    };

    return s_es;
}

// Construction

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string()
    : m_buffer(0)
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(class_type const &rhs)
    : m_buffer(_copy_buffer(rhs.m_buffer))
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(class_type const &rhs, size_type pos)
    : m_buffer(_alloc_buffer(rhs.m_buffer + pos))
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(class_type const &rhs, size_type pos, size_type cch)
    : m_buffer(_alloc_buffer(rhs.m_buffer + pos, cch))
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(char_type const *s) // Not, not explicit. Sigh
    : m_buffer(_alloc_buffer(s))
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(char_type const *s, size_type cch)
    : m_buffer(_alloc_buffer(s, cch))
{
    stlsoft_assert(_is_valid());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(ss_typename_type_k basic_simple_string<C, T, A>::size_type n, ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
    : m_buffer(0)
{
    stlsoft_assert(_is_valid());

    assign(n, ch);
}

#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::basic_simple_string(basic_simple_string<C, T, A>::char_type const *first, basic_simple_string<C, T, A>::char_type const *last)
    : m_buffer(_alloc_buffer(first, last - first))
{
    stlsoft_assert(_is_valid());
}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline basic_simple_string<C, T, A>::~basic_simple_string()
{
    stlsoft_assert(_is_valid());

    if(m_buffer != 0)
    {
        _destroy_buffer(m_buffer);
    }
}

// Comparison

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline /* static */ ss_sint_t basic_simple_string<C, T, A>::_compare(ss_typename_type_k basic_simple_string<C, T, A>::value_type const *lhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type lhs_len, ss_typename_type_k basic_simple_string<C, T, A>::value_type const *rhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type rhs_len)
{
    size_type   cmp_len =   (lhs_len < rhs_len) ? lhs_len : rhs_len;
    ss_int_t    result  =   traits_type::compare(lhs, rhs, cmp_len);

    if(result == 0)
    {
        result = static_cast<ss_int_t>(lhs_len) - static_cast<ss_int_t>(rhs_len);
    }

    return result;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::value_type const *s, ss_typename_type_k basic_simple_string<C, T, A>::size_type cchRhs) const
{
    size_type   lhs_len =   length();

    if(!(pos < lhs_len))
    {
        pos = lhs_len;
    }
    else
    {
        lhs_len -= pos;
    }

    if(cch < lhs_len)
    {
        lhs_len = cch;
    }

    size_type   rhs_len =   (s == 0) ? 0 : traits_type::length(s);

    if(cchRhs < rhs_len)
    {
        rhs_len = cchRhs;
    }

    return _compare(m_buffer + pos, lhs_len, s, rhs_len);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::value_type const *s) const
{
    size_type   lhs_len =   length();

    if(!(pos < lhs_len))
    {
        pos = lhs_len;
    }
    else
    {
        lhs_len -= pos;
    }

    if(cch < lhs_len)
    {
        lhs_len = cch;
    }

    size_type   rhs_len =   (s == 0) ? 0 : traits_type::length(s);

    return _compare(m_buffer + pos, lhs_len, s, rhs_len);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::value_type const *s) const
{
    size_type   lhs_len =   length();
    size_type   rhs_len =   (s == 0) ? 0 : traits_type::length(s);

    return _compare(m_buffer, lhs_len, s, rhs_len);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::class_type const &rhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type posRhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type cchRhs) const
{
    size_type   lhs_len =   length();

    if(!(pos < lhs_len))
    {
        pos = lhs_len;
    }
    else
    {
        lhs_len -= pos;
    }

    if(cch < lhs_len)
    {
        lhs_len = cch;
    }

    size_type   rhs_len =   rhs.length();

    if(!(posRhs < rhs_len))
    {
        posRhs = rhs_len;
    }
    else
    {
        rhs_len -= posRhs;
    }

    if(cchRhs < rhs_len)
    {
        rhs_len = cchRhs;
    }

    return _compare(m_buffer + pos, lhs_len, rhs.m_buffer + posRhs, rhs_len);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::class_type const &rhs) const
{
    size_type   lhs_len =   length();

    if(!(pos < lhs_len))
    {
        pos = lhs_len;
    }
    else
    {
        lhs_len -= pos;
    }

    if(cch < lhs_len)
    {
        lhs_len = cch;
    }

    size_type   rhs_len =   rhs.length();

    return _compare(m_buffer + pos, lhs_len, rhs.m_buffer, rhs_len);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_sint_t basic_simple_string<C, T, A>::compare(ss_typename_type_k basic_simple_string<C, T, A>::class_type const &rhs) const
{
    size_type   lhs_len =   length();
    size_type   rhs_len =   rhs.length();

    return _compare(m_buffer, lhs_len, rhs.m_buffer, rhs_len);
}

// Accessors
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::reference basic_simple_string<C, T, A>::operator [](ss_typename_type_k basic_simple_string<C, T, A>::size_type index)
{
    stlsoft_message_assert("index access out of range in frame_string", index < length() + 1);   // Has to be +1, since legitimate to take address of one-past-the-end

    stlsoft_assert(_is_valid());

    return m_buffer[index];
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::const_reference basic_simple_string<C, T, A>::operator [](ss_typename_type_k basic_simple_string<C, T, A>::size_type index) const
{
    stlsoft_message_assert("index access out of range in frame_string", index < length() + 1);   // Has to be +1, since legitimate to take address of one-past-the-end

    stlsoft_assert(_is_valid());

    return m_buffer[index];
}


template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::value_type const *basic_simple_string<C, T, A>::c_str() const
{
    return (m_buffer == 0) ? _empty_string() : m_buffer;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::value_type const *basic_simple_string<C, T, A>::data() const
{
    return (m_buffer == 0) ? _empty_string() : m_buffer;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::size_type basic_simple_string<C, T, A>::copy(ss_typename_type_k basic_simple_string<C, T, A>::value_type *dest, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::size_type pos /* = 0 */) const
{
    size_type   len =   length();

    if(pos < len)
    {
        if(len < pos + cch)
        {
            cch = len - pos;
        }

        traits_type::copy(dest, data() + pos, cch);
    }
    else
    {
        cch = 0;
    }

    return cch;
}

// Iteration

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::const_iterator basic_simple_string<C, T, A>::begin() const
{
    return const_cast<class_type*>(this)->_begin();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::const_iterator basic_simple_string<C, T, A>::end() const
{
    return const_cast<class_type*>(this)->_end();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::iterator basic_simple_string<C, T, A>::begin()
{
    return _begin();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::iterator basic_simple_string<C, T, A>::end()
{
    return _end();
}

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::const_reverse_iterator basic_simple_string<C, T, A>::rbegin() const
{
    return const_reverse_iterator(end());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::const_reverse_iterator basic_simple_string<C, T, A>::rend() const
{
    return const_reverse_iterator(begin());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::reverse_iterator basic_simple_string<C, T, A>::rbegin()
{
    return reverse_iterator(end());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::reverse_iterator basic_simple_string<C, T, A>::rend()
{
    return reverse_iterator(begin());
}
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// Assignment
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(const ss_typename_type_k basic_simple_string<C, T, A>::char_type *s, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    stlsoft_assert(_is_valid());

    if(m_buffer == 0)
    {
        if(cch == 0)
        {
            // Nothing to do
        }
        else
        {
            m_buffer = _alloc_buffer(s, cch);
        }
    }
    else
    {
        if(s == 0)
        {
            _destroy_buffer(m_buffer);
            m_buffer = 0;
        }
        else
        {
            // Here is an opportunity to optimise a bit. We will do more so in a
            // later release, but for the moment we will reuse our existing
            // buffer if its capacity is sufficient for our purposes

            string_buffer   *buffer =   _buffer_from_ptr(m_buffer);
            size_type       len     =   traits_type::length(s);

            if(len < cch)
            {
                cch = len;
            }

            if(cch < buffer->capacity)
            {
                traits_type::copy(buffer->contents, s, cch);
                buffer->contents[cch] = 0;
                buffer->length = cch;
            }
            else
            {
                char_type   *new_buffer =   _alloc_buffer(s, cch);

                _destroy_buffer(m_buffer);
                m_buffer = new_buffer;
            }
        }
    }

    stlsoft_assert(_is_valid());
    return *this;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(const ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    return assign(s, (s == 0) ? 0 : traits_type::length(s));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(const ss_typename_type_k basic_simple_string<C, T, A>::class_type &rhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    char_type   *s  =   rhs.m_buffer;
    size_type   len =   rhs.length();

    if(len < pos)
    {
        pos = len;
    }

    if(len - pos < cch)
    {
        cch = len - pos;
    }

    if(s != 0)
    {
        s += pos;
    }
    else
    {
        cch = 0;
    }

    return assign(s, cch);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(const ss_typename_type_k basic_simple_string<C, T, A>::class_type &rhs)
{
    return assign(rhs.m_buffer, rhs.length());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
{
    typedef auto_buffer<char_type, allocator_type>  buffer_t;

    buffer_t    buffer(cch);

    (void)for_each_set_value(buffer.begin(), buffer.end(), ch);

    return assign(buffer, buffer.size());
}

#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::assign(ss_typename_type_k basic_simple_string<C, T, A>::const_iterator first, ss_typename_type_k basic_simple_string<C, T, A>::const_iterator last)
{
    // We have to use this strange appearing this, because of Visual C++ .NET's
    // disgusting STL swill. Sigh!
    return assign(&(*first), last - first);
}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type const &basic_simple_string<C, T, A>::operator =(ss_typename_type_k basic_simple_string<C, T, A>::class_type const &rhs)
{
    return assign(rhs);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type const &basic_simple_string<C, T, A>::operator =(ss_typename_type_k basic_simple_string<C, T, A>::char_type const *s)
{
    return assign(s);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type const &basic_simple_string<C, T, A>::operator =(ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
{
    char_type   sz[2] = { ch, traits_type::to_char_type(0) };

    return assign(sz);
}


// Appending
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(const ss_typename_type_k basic_simple_string<C, T, A>::char_type *s, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    stlsoft_assert(_is_valid());

    if(m_buffer == 0)
    {
        assign(s, cch);
    }
    else
    {
        if( s == 0 ||
            cch == 0)
        {
            // Nothing to do
        }
        else
        {
            // We're taking a length here, which may have been done already.
            // This should be optimised out in a subsequent release
            size_type len = traits_type::length(s);

            if(len < cch)
            {
                cch = len;
            }

            string_buffer   *old_buffer =   0;
            string_buffer   *buffer     =   _buffer_from_ptr(m_buffer);
            size_type       buf_len     =   buffer->length;

            if(buffer->capacity - buf_len < 1 + cch)
            {
                // Allocate a new buffer of sufficient size
                char_type   *new_buffer =   _alloc_buffer(buffer->contents, buf_len + cch);

                if(new_buffer == 0) // Some allocators do not throw on failure!
                {
                    cch = 0;
                }
                else
                {
                    old_buffer = buffer;    // Mark for destruction, but hold around in case appending from self
                    m_buffer = new_buffer;
                    buffer = _buffer_from_ptr(new_buffer);
                }
            }

            traits_type::copy(buffer->contents + buf_len, s, cch);
            buffer->length += cch;
            buffer->contents[buffer->length] = traits_type::to_char_type(0);

            if(old_buffer != 0)
            {
                _destroy_buffer(old_buffer);
            }
        }
    }

    stlsoft_assert(_is_valid());
    return *this;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(const ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    return append(s, (s == 0) ? 0 : traits_type::length(s));
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(const ss_typename_type_k basic_simple_string<C, T, A>::class_type &rhs, ss_typename_type_k basic_simple_string<C, T, A>::size_type pos, ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    char_type   *s  =   rhs.m_buffer;
    size_type   len =   rhs.length();

    if(len < pos)
    {
        pos = len;
    }

    if(len - pos < cch)
    {
        cch = len - pos;
    }

    if(s != 0)
    {
        s += pos;
    }
    else
    {
        cch = 0;
    }

    return append(s, cch);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(const ss_typename_type_k basic_simple_string<C, T, A>::class_type &str)
{
    return append(str.m_buffer, str.length());
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(ss_typename_type_k basic_simple_string<C, T, A>::size_type cch, ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
{
    if(m_buffer == 0)
    {
        assign(cch, ch);
    }
    else
    {
        typedef auto_buffer<char_type, allocator_type>  buffer_t;

        buffer_t    buffer(cch);

        (void)for_each_set_value(buffer.begin(), buffer.end(), ch);

        append(buffer, buffer.size());
    }

    return *this;
}

#if !defined(__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT)
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::append(ss_typename_type_k basic_simple_string<C, T, A>::const_iterator first, ss_typename_type_k basic_simple_string<C, T, A>::const_iterator last)
{
    // We have to use this strange appearing this, because of Visual C++ .NET's
    // disgusting STL swill. Sigh!
    return append(&(*first), last - first);
}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_RANGE_METHOD_SUPPORT */

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::operator +=(ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
{
    return append(1, ch);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::operator +=(const ss_typename_type_k basic_simple_string<C, T, A>::char_type *s)
{
    return append(s);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::class_type &basic_simple_string<C, T, A>::operator +=(const ss_typename_type_k basic_simple_string<C, T, A>::class_type &rhs)
{
    return append(rhs);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline void basic_simple_string<C, T, A>::push_back(ss_typename_type_k basic_simple_string<C, T, A>::char_type ch)
{
    append(1, ch);
}

// Operations
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline void basic_simple_string<C, T, A>::reserve(ss_typename_type_k basic_simple_string<C, T, A>::size_type cch)
{
    if(length() < cch)
    {
        if(m_buffer == 0)
        {
            m_buffer = _alloc_buffer(_empty_string(), cch);
        }
        else
        {
            // Allocate a new buffer of sufficient size
            char_type   *new_buffer =   _alloc_buffer(c_str(), cch);

            if(new_buffer != 0) // Some allocators do not throw on failure!
            {
                _destroy_buffer(m_buffer);
                m_buffer = new_buffer;
            }
        }
    }
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline void basic_simple_string<C, T, A>::swap(ss_typename_type_k basic_simple_string<C, T, A>::class_type &other)
{
    stlsoft_assert(_is_valid());
    stlsoft_assert(other._is_valid());

    char_type   *buffer =   other.m_buffer;

    other.m_buffer  =   m_buffer;
    m_buffer        =   buffer;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline void basic_simple_string<C, T, A>::clear()
{
    if(m_buffer != 0)
    {
        string_buffer   *buffer =   _buffer_from_ptr(m_buffer);

        buffer->length      =   0;
        buffer->contents[0] =   traits_type::to_char_type(0);
    }
}

// Attributes
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::size_type basic_simple_string<C, T, A>::size() const
{
    stlsoft_assert(_is_valid());

    return (m_buffer == 0) ? 0 : _buffer_from_ptr(const_cast<char_type const*>(m_buffer))->length;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::size_type basic_simple_string<C, T, A>::max_size() const
{
    stlsoft_assert(_is_valid());

    return static_cast<size_type>(-1) / sizeof(char_type);
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::size_type basic_simple_string<C, T, A>::length() const
{
    stlsoft_assert(_is_valid());

    return size();
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_typename_type_k basic_simple_string<C, T, A>::size_type basic_simple_string<C, T, A>::capacity() const
{
    stlsoft_assert(_is_valid());

    return (m_buffer == 0) ? 0 : _buffer_from_ptr(const_cast<char_type const*>(m_buffer))->capacity;
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_bool_t basic_simple_string<C, T, A>::empty() const
{
    stlsoft_assert(_is_valid());

    return length() == 0;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * String access shims
 */

/* c_str_ptr_null */

/// \brief Returns the corresponding C-string pointer of \c s, or a null pointer
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline C const *c_str_ptr_null(basic_simple_string<C, T, A> const &s)
{
    return (s.length() == 0) ? 0 : s.c_str();
}

/* c_str_ptr */

/// \brief Returns the corresponding C-string pointer of \c s
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline C const *c_str_ptr(basic_simple_string<C, T, A> const &s)
{
    return s.c_str();
}

/* c_str_ptr_len */

/// \brief Returns the length (in characters) of \c s, <b><i>not</i></b> including the null-terminating character
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_size_t c_str_len(basic_simple_string<C, T, A> const &s)
{
    return s.length();
}

/* c_str_ptr_size */

/// \brief Returns the size (in bytes) of the contents of \c s, <b><i>not</i></b> including the null-terminating character
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        ,   ss_typename_param_k A
        >
inline ss_size_t c_str_size(basic_simple_string<C, T, A> const &s)
{
    return c_str_len(s) * sizeof(C);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group stlsoft_string_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_SIMPLE_STRING */

/* ////////////////////////////////////////////////////////////////////////// */
