/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_true_typedef.h
 *
 * Purpose:     Contains the auto_destructor and auto_array_destructor template
 *              classes.
 *
 * Created:     16th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_TRUE_TYPEDEF
#define _STLSOFT_INCL_H_STLSOFT_TRUE_TYPEDEF

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_TRUE_TYPEDEF_MAJOR       1
#define _STLSOFT_VER_H_STLSOFT_TRUE_TYPEDEF_MINOR       5
#define _STLSOFT_VER_H_STLSOFT_TRUE_TYPEDEF_REVISION    5
#define _STLSOFT_VER_H_STLSOFT_TRUE_TYPEDEF_EDIT        41
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/* U is the unique type, and is usually a type generated by the
 * stlsoft_gen_opaque() macro, as in
 *
 *  stlsoft_gen_opaque(h_uniquelong_tt)
 *  typedef true_typedef<long, h_uniquelong_tt>     uniquelong_tt;
 *
 * thus generating a unique type uniquelong over base-type long.
 */

/// This class is used to create strong types from base types, to facilitate
/// type disambiguation, overloading and mutual incompatibility.
///
/// For example, the following code creates two types based on the \c int type:
///
/// &nbsp;&nbsp;<code>stlsoft_gen_opaque(h_protocol_tt)</code><br>
/// &nbsp;&nbsp;<code>stlsoft_gen_opaque(h_port_tt)</code><br>
/// &nbsp;&nbsp;<code>typedef true_typedef<long, h_protocol_tt>  protocol_tt;</code><br>
/// &nbsp;&nbsp;<code>typedef true_typedef<long, h_port_tt>      port_tt;</code><br>
/// <br>
/// &nbsp;&nbsp;<code>protocol_tt pro1(3);    // Ok</code><br>
/// &nbsp;&nbsp;<code>port_tt     prt1(8088); // Ok</code><br>
/// &nbsp;&nbsp;<code>protocol_tt pro2(pro1); // Ok, can copy from another protocol_tt</code><br>
/// &nbsp;&nbsp;<code>port_tt     prt2(pro2); // Error, cannot copy from a protocol_tt</code><br>
///
///
/// \param T The base type, e.g. \c int, \c std::wstring
/// \param U The unique type, usually created by using stlsoft_gen_opaque()

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
class true_typedef
{
public:
    /// The value type
    typedef T                   value_type;
    /// The unique type
    typedef U                   unique_type;
    /// The type of the current parameterisation
    typedef true_typedef<T, U>  class_type;
    /// The reference type
    typedef T                   &reference;
    /// The non-mutating (const) reference type
    typedef T const             &const_reference;

// Construction
public:
    /// Default constructor
    true_typedef()
        : m_value(value_type())
    {}
    /// Construct from a value type instance
    ss_explicit_k true_typedef(value_type const &value)
        : m_value(value)
    {}
    /// Copy constructor
    true_typedef(class_type const &rhs)
        : m_value(rhs.m_value)
    {}

    /// Copy assignment operator
    class_type const &operator =(class_type const &rhs)
    {
        m_value = rhs.m_value;

        return *this;
    }

// Accessors
public:
    /// Provides non-mutating (const) access to the base type value
    const_reference base_type_value() const
    {
        return m_value;
    }
    /// Provides mutating access to the base type value
    reference base_type_value()
    {
        return m_value;
    }

// Members
private:
    value_type  m_value;

// Not to be implemented
private:
    // Not provided, as the syntax is less ambiguous when
    // assignment from an explicit temporary is made
    class_type const &operator =(value_type const &value);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

// Pre-increment

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> &operator ++(true_typedef<T, U> &v)
{
    ++v.base_type_value();

    return v;
}

// Post-increment

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const operator ++(true_typedef<T, U> &v, int)
{
    true_typedef<T, U>  r(v);

    v.base_type_value()++;

    return r;
}

// Pre-decrement

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> &operator --(true_typedef<T, U> &v)
{
    --v.base_type_value();

    return v;
}

// Post-decrement

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const operator --(true_typedef<T, U> &v, int)
{
    true_typedef<T, U>  r(v);

    v.base_type_value()--;

    return r;
}

// operator ==

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator ==(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return lhs.base_type_value() == rhs.base_type_value();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator ==(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator ==(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.base_type_value() == rhs;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator ==(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator ==(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs == rhs.base_type_value();
}


// operator !=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator !=(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return ! operator ==(lhs, rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator !=(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator !=(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return ! operator ==(lhs, rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator !=(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator !=(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return ! operator ==(lhs, rhs);
}

// operator <

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator <(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return lhs.base_type_value() < rhs.base_type_value();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator <(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.base_type_value() < rhs;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator <(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs < rhs.base_type_value();
}

// operator <=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator <=(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return lhs.base_type_value() <= rhs.base_type_value();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <=(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator <=(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.base_type_value() <= rhs;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator <=(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator <=(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs <= rhs.base_type_value();
}

// operator >

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator >(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return lhs.base_type_value() > rhs.base_type_value();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator >(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.base_type_value() > rhs;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator >(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs > rhs.base_type_value();
}

// operator >=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline ss_bool_t operator >=(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return lhs.base_type_value() >= rhs.base_type_value();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >=(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline ss_bool_t operator >=(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs.base_type_value() >= rhs;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline ss_bool_t operator >=(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline ss_bool_t operator >=(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return lhs >= rhs.base_type_value();
}

// operator +

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator +(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() + rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator +(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator +(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() + rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator +(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator +(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs + rhs.base_type_value());
}

// operator -

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator -(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() - rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator -(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator -(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() - rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator -(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator -(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs - rhs.base_type_value());
}

// operator *

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator *(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() * rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator *(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator *(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() * rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator *(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator *(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs * rhs.base_type_value());
}

// operator /

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator /(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() / rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator /(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator /(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() / rhs);
}


template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator /(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator /(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs / rhs.base_type_value());
}

// operator %

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator %(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() % rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator %(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator %(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() % rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator %(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator %(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs % rhs.base_type_value());
}

// operator ^

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator ^(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() ^ rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator ^(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator ^(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() ^ rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator ^(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator ^(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs ^ rhs.base_type_value());
}

// operator ~

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator ~(true_typedef<T, U> const &v)
{
    return true_typedef<T, U>(~v.base_type_value());
}

// operator <<

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator <<(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() << rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator <<(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator <<(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() << rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator <<(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator <<(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs << rhs.base_type_value());
}

// operator >>

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator >>(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() >> rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator >>(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator >>(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() >> rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator >>(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator >>(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs >> rhs.base_type_value());
}

// operator &

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator &(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() & rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator &(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator &(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() & rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator &(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator &(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs & rhs.base_type_value());
}

// operator |

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> operator |(true_typedef<T, U> const &lhs, true_typedef<T, U> const &rhs)
{
    return true_typedef<T, U>(lhs.base_type_value() | rhs.base_type_value());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator |(true_typedef<T, U> const &lhs, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> operator |(true_typedef<T, U> const &lhs, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs.base_type_value() | rhs);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> operator |(ss_typename_type_k true_typedef<T, U>::value_type const &lhs, true_typedef<T, U> const &rhs)
#else
inline true_typedef<T, U> operator |(T const &lhs, true_typedef<T, U> const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    return true_typedef<T, U>(lhs | rhs.base_type_value());
}

// operator +=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator +=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator +=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() += rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator +=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() += rhs.base_type_value();

    return v;
}

// operator -=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator -=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator -=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() -= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator -=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() -= rhs.base_type_value();

    return v;
}

// operator *=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator *=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator *=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() *= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator *=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() *= rhs.base_type_value();

    return v;
}

// operator /=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator /=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator /=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() /= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator /=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() /= rhs.base_type_value();

    return v;
}

// operator %=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator %=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator %=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() %= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator %=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() %= rhs.base_type_value();

    return v;
}

// operator ^=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator ^=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator ^=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() ^= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator ^=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() ^= rhs.base_type_value();

    return v;
}

// operator <<=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator <<=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator <<=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() <<= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator <<=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() <<= rhs.base_type_value();

    return v;
}

// operator >>=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator >>=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator >>=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() >>= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator >>=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() >>= rhs.base_type_value();

    return v;
}

// operator &=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator &=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator &=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() &= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator &=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() &= rhs.base_type_value();

    return v;
}

// operator |=

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
#ifdef __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT
inline true_typedef<T, U> const &operator |=(true_typedef<T, U> &v, ss_typename_type_k true_typedef<T, U>::value_type const &rhs)
#else
inline true_typedef<T, U> const &operator |=(true_typedef<T, U> &v, T const &rhs)
#endif /* __STLSOFT_CF_TEMPLATE_OUTOFCLASSFN_QUALIFIED_TYPE_SUPPORT */
{
    v.base_type_value() |= rhs;

    return v;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k U>
inline true_typedef<T, U> const &operator |=(true_typedef<T, U> &v, true_typedef<T, U> const &rhs)
{
    v.base_type_value() |= rhs.base_type_value();

    return v;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_TRUE_TYPEDEF */

/* ////////////////////////////////////////////////////////////////////////// */
