/* /////////////////////////////////////////////////////////////////////////////
 * File:		string_access_test.cpp
 *
 * Purpose:		Illustrates the string access components of the MFCSTL libraries
 *
 * Created:		5th December 2002
 * Updated:		30th June 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2002. All rights reserved.
 *
 * Home:		http://www.winstl.org/
 *
 * ////////////////////////////////////////////////////////////////////////// */

// This will cause various compile-time messages to be emitted. When you get
// sick of them just comment out or remove this #define
#define	_STLSOFT_COMPILE_VERBOSE

#include <mfcstl.h>

#include <stlsoft_string_access.h>

stlsoft_ns_using(c_str_ptr)
stlsoft_ns_using(c_str_len)

#include <mfcstl_string_access.h>

mfcstl_ns_using(c_str_ptr)
mfcstl_ns_using(c_str_len)

#include "resource.h"

////////////////////////////////////////////////////////////////////////////////
// Classes

// class CMyApp
class CMyApp
	: public CWinApp
{
public:
	virtual BOOL InitInstance();
};

// class CMyFrame
class CMyFrame
	: public CFrameWnd
{
public:
	CMyFrame();

protected:
	afx_msg void OnPaint();

	DECLARE_MESSAGE_MAP()
};

////////////////////////////////////////////////////////////////////////////////

template <ms_typename_param_k S>
void put_to_dc(CDC &dc, POINT &origin, S const &s)
{
	int		cch	=	c_str_len(s);
	SIZE	cxy	=	dc.GetTextExtent(c_str_ptr(s), cch);

	dc.TextOut(origin.x, origin.y, c_str_ptr(s));

	origin.y += 1 + cxy.cy;
}

////////////////////////////////////////////////////////////////////////////////

CMyApp gMainApp;

////////////////////////////////////////////////////////////////////////////////

BOOL CMyApp::InitInstance()
{
	m_pMainWnd = new CMyFrame();
	m_pMainWnd->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

BEGIN_MESSAGE_MAP(CMyFrame, CFrameWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CMyFrame::CMyFrame()
{
	Create(NULL, "MFCSTL - string_access_test");
}

void CMyFrame::OnPaint()
{
	CPaintDC dc( this );
	CPoint	origin(1, 1);

	// Here the usefulness of the string access shims is demonstrated
	put_to_dc(dc, origin, "A C-style literal string");

	put_to_dc(dc, origin, CString("A CString, made from a literal string"));

	CString	strText;
	strText.LoadString(IDS_RESOURCE);
	put_to_dc(dc, origin, strText);
}

////////////////////////////////////////////////////////////////////////////////
