/* /////////////////////////////////////////////////////////////////////////////
 * File:        unixstl_readdir_sequence.h
 *
 * Purpose:     readdir_sequence class.
 *
 * Created:     15th January 2002
 * Updated:     14th August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/unixstl
 *                          http://www.unixstl.org/
 *
 *              email:      submissions@unixstl.org  for submissions
 *                          admin@unixstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _INCL_UNIXSTL_H_UNIXSTL_READDIR_SEQUENCE
#define _INCL_UNIXSTL_H_UNIXSTL_READDIR_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _UNIXSTL_VER_H_UNIXSTL_READDIR_SEQUENCE_MAJOR       1
#define _UNIXSTL_VER_H_UNIXSTL_READDIR_SEQUENCE_MINOR       3
#define _UNIXSTL_VER_H_UNIXSTL_READDIR_SEQUENCE_REVISION    5
#define _UNIXSTL_VER_H_UNIXSTL_READDIR_SEQUENCE_EDIT        34
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _INCL_UNIXSTL_H_UNIXSTL
 #include "unixstl.h"               // Include the UNIXSTL root header
#endif /* !_INCL_UNIXSTL_H_UNIXSTL */
#ifndef _INCL_UNIXSTL_H_UNIXSTL_LIMITS
 #include "unixstl_limits.h"        // UNIXSTL_NAME_MAX
#endif /* !_INCL_UNIXSTL_H_UNIXSTL_LIMITS */
#ifndef _STLSOFT_INCL_H_STLSOFT_FRAME_STRING
 #include "stlsoft_frame_string.h"  // stlsoft::basic_frame_string
#endif /* !_STLSOFT_INCL_H_STLSOFT_FRAME_STRING */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */

#include <sys/types.h>
#include <dirent.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The UNIXSTL components are contained within the unixstl namespace. This is
 * actually an alias for stlsoft::unixstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _UNIXSTL_NO_NAMESPACE   unixstl definition
 * ---------------------    ---------------------   -----------------
 *  not defined              not defined             = stlsoft::unixstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             unixstl
 *  defined                  defined                 not defined
 *
 */

/* No STLSoft namespaces means no UNIXSTL namespaces */
#ifdef _STLSOFT_NO_NAMESPACES
 #define _UNIXSTL_NO_NAMESPACES
#endif /* _STLSOFT_NO_NAMESPACES */

/* No UNIXSTL namespaces means no unixstl namespace */
#ifdef _UNIXSTL_NO_NAMESPACES
 #define _UNIXSTL_NO_NAMESPACE
#endif /* _UNIXSTL_NO_NAMESPACES */

#ifndef _UNIXSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::unixstl */
namespace unixstl
{
 #else
/* Define stlsoft::unixstl_project */

namespace stlsoft
{

namespace unixstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_UNIXSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \defgroup unixstl_filesystem_library File-System Library (UNIXSTL)
/// \ingroup UNIXSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating UNIX file-system objects
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// \brief Iterator for readdir_sequence class
///
/// This class performs as a non-mutating iterator (aka const iterator) for the
/// readdir_sequence class.
///
///

class readdir_sequence_const_iterator
    : public stlsoft_ns_qual(iterator_base) <   unixstl_ns_qual_std(input_iterator_tag)
                                            ,   struct dirent const *
                                            ,   us_ptrdiff_t
                                            ,   struct dirent const **
                                            ,   struct dirent const *&
                                            >
{
public:
    /// The class type
    typedef readdir_sequence_const_iterator             class_type;
    /// The type on the rhs of <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">move</a> expressions
    typedef stlsoft_define_move_rhs_type(class_type)    rhs_type;
    /// The value type
    typedef struct dirent const                         *value_type;
//    typedef value_type                                  *pointer;
//    typedef value_type                                  &reference;

// Construction
private:
    friend class readdir_sequence;

    /// Construct an instance and begin a sequence iteration on the given dir.
    readdir_sequence_const_iterator(DIR *dir, us_bool_t bIgnoreDots)
        : m_dir(dir)
        , m_entry(0)
        , m_bIgnoreDots(bIgnoreDots)
    {
        if(m_dir != 0)
        {
            operator ++();
        }
    }
public:
    /// Default constructor
    readdir_sequence_const_iterator()
        : m_dir(0)
        , m_entry(0)
    {}
    /// Constructs an instance based on the given directory entry
    readdir_sequence_const_iterator(dirent *entry)
        : m_dir(0)
        , m_entry(entry)
    {}
    /// <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">Move constructor</a>
    readdir_sequence_const_iterator(rhs_type rhs)
        : m_dir(rhs.m_dir)
        , m_entry(rhs.m_entry)
        , m_bIgnoreDots(rhs.m_bIgnoreDots)
    {
        move_lhs_from_rhs(rhs).m_dir   = 0;
        move_lhs_from_rhs(rhs).m_entry = 0;
    }
    /// Release the search handle
    ~readdir_sequence_const_iterator() unixstl_throw_0()
    {
        if(m_dir != 0)
        {
            closedir(m_dir);
        }
    }

    /// <a href = "http://synesis.com.au/resources/articles/cpp/movectors.pdf">Move assignment</a> operator
    class_type const &operator =(rhs_type rhs)
    {
        m_dir           =   rhs.m_dir;
        m_entry         =   rhs.m_entry;
        m_bIgnoreDots   =   rhs.m_bIgnoreDots;

        move_lhs_from_rhs(rhs).m_dir   = 0;
        move_lhs_from_rhs(rhs).m_entry = 0;

        return *this;
    }

// Accessors
public:
    /// Returns the value representative 
    value_type operator *() const
    {
        unixstl_assert(m_entry != 0);

        return m_entry;
    }

    /// Moves the iteration on to the next point in the sequence, or end() if
    /// the sequence is exhausted
    class_type &operator ++()
    {
        unixstl_assert(m_dir != 0);

        m_entry = readdir(m_dir);

        if(m_bIgnoreDots)
        {
            for(; m_entry != 0; m_entry = readdir(m_dir))
            {
                unixstl_assert(m_entry->d_name);
                if(m_entry->d_name[0] != '.')
                {
                    break;
                }
                if(m_entry->d_name[1] != '\0')
                {
                    if( m_entry->d_name[1] != '.' &&
                        m_entry->d_name[2] != '\0')
                    {
                        break;
                    }
                }
            }
        }

        if(m_entry == 0)
        {
            closedir(m_dir);
            m_dir = 0;
        }

        return *this;
    }
    /// Post-increment form of operator ++().
    ///
    /// \note Because this version uses a temporary on which to call the
    /// pre-increment form it is thereby less efficient, and should not be used
    /// except where post-increment semantics are required.
    class_type operator ++(int)
    {
        class_type  ret(m_entry);

        operator ++();

        return ret;
    }

    /// Compares \c this for equality with \c rhs
    ///
    /// \retval true if the iterators are equivalent
    /// \retval false if the iterators are not equivalent
    bool operator ==(class_type const &rhs) const
    {
        unixstl_assert(m_dir == 0 || rhs.m_dir == 0 || m_dir == rhs.m_dir);

        return m_entry == rhs.m_entry;
    }
    /// Compares \c this for inequality with \c rhs
    ///
    /// \retval false if the iterators are equivalent
    /// \retval true if the iterators are not equivalent
    bool operator !=(class_type const &rhs) const
    {
        return !operator ==(rhs);
    }

// Members
private:
    DIR             *m_dir;
    struct dirent   *m_entry;
    us_bool_t       m_bIgnoreDots;

// Not to be implemented
private:
};


/// \brief STL-like readonly sequence based on directory contents
///
/// This class presents and STL-like readonly sequence interface to allow the 
/// iteration over the contents of a directory.

class readdir_sequence
{
private:
    typedef readdir_sequence                                    class_type;
public:
//    typedef us_size_t                                           size_type;
    typedef readdir_sequence_const_iterator                     const_iterator;
    typedef readdir_sequence_const_iterator::value_type         value_type;
//    typedef readdir_sequence_const_iterator::pointer            pointer;
//    typedef readdir_sequence_const_iterator::pointer const      const_pointer;
//    typedef readdir_sequence_const_iterator::reference          reference;
    typedef readdir_sequence_const_iterator::reference const    const_reference;

private:
    typedef stlsoft_ns_qual(basic_frame_string) <   us_char_a_t
                                                ,   UNIXSTL_NAME_MAX
                                                >               string_type;

public:
    enum
    {
            includeDots =   0x0008  /*!< Requests that dots directories be included in the returned sequence */
        ,
    };

// Construction
public:
    /// \brief Constructs a sequence according to the given criteria
    ///
    /// The constructor initialises a readdir_sequence instance on the given
    /// directory with the given flags.
    ///
    /// \param name The directory whose contents are to be searched
    /// \param flags Flags to alter the behaviour of the search
    readdir_sequence(us_char_a_t const *name, us_int_t flags = 0)
        : m_name(name)
        , m_flags(_validate_flags(flags))
    {}

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const
    {
        DIR *dir = opendir(m_name.c_str());

        return const_iterator(dir, (m_flags & includeDots) == 0);
    }
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const
    {
        return const_iterator();
    }

// Attributes
public:
#if 0
    /// Returns the number of elements in the sequence
    ///
    /// \note Nor currently implemented
    size_type   size() const
    {
        return 0;
    }
#endif /* 0 */

    /// \brief Indicates whether the search sequence is empty
    us_bool_t empty() const
    {
        return begin() != end();
    }

// Implementation
private:
    /// \brief Ensures that the flags are correct
    static us_int_t _validate_flags(us_int_t flags)
    {
        return flags & includeDots;
    }

// Members
private:
    string_type m_name;
    us_int_t    m_flags;
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group unixstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _UNIXSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace unixstl
 #else
} // namespace unixstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_UNIXSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_INCL_UNIXSTL_H_UNIXSTL_READDIR_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
