/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_current_directory_scope.h (formerly MLPwdScp.h, ::SynesisStd)
 *
 * Purpose:     Current working directory scoping class.
 *
 * Created:     12th November 1998
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_CURRENT_DIRECTORY_SCOPE
#define _WINSTL_INCL_H_WINSTL_CURRENT_DIRECTORY_SCOPE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_CURRENT_DIRECTORY_SCOPE_MAJOR      2
#define _WINSTL_VER_H_WINSTL_CURRENT_DIRECTORY_SCOPE_MINOR      3
#define _WINSTL_VER_H_WINSTL_CURRENT_DIRECTORY_SCOPE_REVISION   2
#define _WINSTL_VER_H_WINSTL_CURRENT_DIRECTORY_SCOPE_EDIT       52
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
 #include "winstl_filesystem_traits.h"  // file_traits
#endif /* !_WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */
#ifndef _STLSOFT_INCL_H_STLSOFT_STRING_ACCESS
 #include "stlsoft_string_access.h"     // stlsoft::c_str_ptr
#endif /* !_STLSOFT_INCL_H_STLSOFT_STRING_ACCESS */
#ifndef _WINSTL_INCL_H_WINSTL_STRING_ACCESS
 #include "winstl_string_access.h"      // winstl::c_str_ptr
#endif /* !_WINSTL_INCL_H_WINSTL_STRING_ACCESS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * basic_current_directory_scope
 *
 * This class pushes the given directory as the current directory upon
 * construction, and pops back to the original at destruction.
 */

/// \brief Current directory scoping class
///
/// This class scopes the process's current directory, by changing to the path
/// given in the constructor, and then, if that succeeded, changing back in the
/// destructor
///
/// \param C The character type (e.g. \c char, \c wchar_t)
/// \param T The file-system traits. In translators that support default template parameters that defaults to \c filesystem_traits<C>

template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = filesystem_traits<C>
#else
        ,   ws_typename_param_k T /* = filesystem_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_current_directory_scope
{
public:
    typedef C                               char_type;
    typedef T                               traits_type;
    typedef basic_current_directory_scope<C, T>   class_type;
    typedef ws_size_t                       size_type;

// Construction
public:
    /// \brief Constructs a scope instance and changes to the given directory
    ///
    /// \param dir The name of the directory to change the current directory to
    ws_explicit_k basic_current_directory_scope(char_type const *dir)
    {
        _init(c_str_ptr(dir));
    }
#if defined(__STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT)
    /// \brief Constructs a scope instance and changes to the given directory
    ///
    /// \param dir The name of the directory to change the current directory to
    template <ws_typename_param_k S>
    ws_explicit_k basic_current_directory_scope(S const &dir)
    {
        _init(c_str_ptr(dir));
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
    /// \brief Returns the current directory to its original location
    ~basic_current_directory_scope() winstl_throw_0()
    {
        if(m_previous[0] != '\0')
        {
            traits_type::set_current_directory(m_previous);
        }
    }

// Conversions
public:
    /// Returns a C-string pointer to the original directory
    operator char_type const *() const
    {
        return m_previous;
    }

    /// Indicates whether the construction was successful
    ///
    /// \retval true The scope instance was successfully constructed and the current directory changed as per the constructor argument
    /// \retval false The scope instance was not successfully constructed, and the current directory was unchanged.
    operator ws_bool_t () const
    {
        return m_previous[0] != '\0';
    }

    /// Indicates whether the construction failed
    ///
    /// This method is the opposite of operator us_bool_t(), and the return values are inverted.
    ws_bool_t operator !() const
    {
        return !operator ws_bool_t();
    }

// Implementation
private:
    void _init(char_type const *dir)
    {
        if( 0 == traits_type::get_current_directory(winstl_num_elements(m_previous), m_previous) ||
            !traits_type::set_current_directory(dir))
        {
            m_previous[0] = '\0';
        }
    }

// Members
private:
    char_type   m_previous[1 + _MAX_PATH];

// Not to be implemented
private:
    basic_current_directory_scope();
    basic_current_directory_scope(class_type const &);
    class_type const &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs for commonly encountered types
 */

/// Instantiation of the basic_current_directory_scope template for the ANSI character type \c char
typedef basic_current_directory_scope<ws_char_a_t, filesystem_traits<ws_char_a_t> >     current_directory_scope_a;
/// Instantiation of the basic_current_directory_scope template for the Unicode character type \c wchar_t
typedef basic_current_directory_scope<ws_char_w_t, filesystem_traits<ws_char_w_t> >     current_directory_scope_w;
/// Instantiation of the basic_current_directory_scope template for the Win32 character type \c TCHAR
typedef basic_current_directory_scope<TCHAR, filesystem_traits<TCHAR> >                 current_directory_scope;

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_CURRENT_DIRECTORY_SCOPE */

/* ////////////////////////////////////////////////////////////////////////// */
