/* ////////////////////////////////////////////////////////////////////////////
 * File:        winstl_environment_sequence.h
 *
 * Purpose:     environment_sequence class.
 *
 * Created:     31st December 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_ENVIRONMENT_SEQUENCE
#define _WINSTL_INCL_H_WINSTL_ENVIRONMENT_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_ENVIRONMENT_SEQUENCE_MAJOR     1
#define _WINSTL_VER_H_WINSTL_ENVIRONMENT_SEQUENCE_MINOR     1
#define _WINSTL_VER_H_WINSTL_ENVIRONMENT_SEQUENCE_REVISION  2
#define _WINSTL_VER_H_WINSTL_ENVIRONMENT_SEQUENCE_EDIT      11
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"              // const_reverse_iterator_base
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"           // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // processheap_allocator
#endif /* _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */

#include <algorithm>

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_system System Library
/// \ingroup libraries
/// \brief This library provides facilities for accessing system attributes

/// \defgroup winstl_system_library System Library (WinSTL)
/// \ingroup WinSTL libraries_system
/// \brief This library provides facilities for accessing Win32 system attributes
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// STL-like sequence for the system environment variables
///
/// \param C The character type
template <ws_typename_param_k C>
class environment_sequence
{
public:
    /// The character type
    typedef C                                                   char_type;
    /// Structure representing the items
    struct symbol
    {
        /// The symbol name
        char_type const *name;
        /// The symbol value
        char_type const *value;
    };
    /// The value type
    typedef symbol                                              value_type;
    /// The current parameterisation of the type
    typedef environment_sequence<C>                             class_type;
    /// The non-mutable (const) pointer type
    typedef const value_type                                    *const_pointer;
    /// The non-mutable (const) reference type
    typedef const value_type                                    &const_reference;
    /// The size type
    typedef size_t                                              size_type;
    /// The difference type
    typedef ws_ptrdiff_t                                        difference_type;
    /// The non-mutating (const) iterator type
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
      ws_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
        stlsoft_ns_qual(pointer_iterator)   <   value_type
                                            ,   const_pointer
                                            ,   const_reference
                                            >::iterator_type    const_iterator;
    /// The non-mutating (const) reverse iterator type
    typedef stlsoft_ns_qual(const_reverse_iterator_base)<   const_iterator
                                                        ,   value_type
                                                        ,   const_reference
                                                        ,   const_pointer
                                                        ,   difference_type
                                                        >       const_reverse_iterator;

// Construction
public:
    /// Default constructor
    environment_sequence();
    /// Destructor, which releases any resources acquired
    ~environment_sequence() winstl_throw_0();

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const;
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const;

    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    const_reverse_iterator  rbegin() const;
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    const_reverse_iterator  rend() const;

// Iteration
public:
    /// Returns the number of elements in the enviroment sequence
    size_type size() const;
    /// Indicates whether the enviroment sequence is empty
    ws_bool_t empty() const;

// Accessors
public:
    /// Returns the name-value item for the given index
    value_type operator [](size_type index) const;

// Implementation
private:
    typedef auto_buffer<char_type, processheap_allocator<char_type> >   environment_buffer_t;
    typedef auto_buffer<symbol, processheap_allocator<symbol> >         symbols_buffer_t;

    static size_t   _calc_items(char_type const *p, char_type const **q);
    static void     _prepare_items(symbols_buffer_t &symbols, environment_buffer_t &environment, char_type *p, char_type *q);
private:
    static int      _compare_symbol(symbol const &lhs, symbol const &rhs);

public:
    /// Function object that compares environment symbols for the environment_sequence class
    struct compare_symbol
    {
        /// Function call operator, which returns \c true if \c lhs is lexicographically less than \c rhs
        ws_bool_t operator ()(symbol const &lhs, symbol const &rhs)
        {
            return _compare_symbol(lhs, rhs) < 0;
        }
    };

private:
    static char_type const *get_environment_strings();
    static void free_environment_strings(char_type *);

private:
    C const                 *m_p;
    C const                 *m_q;
    symbols_buffer_t        m_symbols;
    environment_buffer_t    m_environment;

// Not to be implemented
private:
    environment_sequence(class_type const &);
    class_type const &operator =(class_type const &);
};

////////////////////////////////////////////////////////////////////////////////
// Implementation

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k C>
inline /* static */ ws_typename_type_k environment_sequence<C>::size_type environment_sequence<C>::_calc_items(ws_typename_type_k environment_sequence<C>::char_type const *p, ws_typename_type_k environment_sequence<C>::char_type const **q)
{
    size_type   c;

    for(c = 0;; ++p)
    {
        if(*p == 0)
        {
            ++c;

            if(*(p + 1) == 0)
            {
                *q = p + 1;
                break;
            }
        }
    }

    return c;
}


template <ws_typename_param_k C>
inline /* static */ void environment_sequence<C>::_prepare_items(ws_typename_type_k environment_sequence<C>::symbols_buffer_t &buffer, ws_typename_type_k environment_sequence<C>::environment_buffer_t &environment, ws_typename_type_k environment_sequence<C>::char_type *p, ws_typename_type_k environment_sequence<C>::char_type *q)
{
    environment_buffer_t::iterator  env_begin   =   environment.begin();
    symbols_buffer_t::iterator      sym_begin   =   buffer.begin();
    char_type                       *begin      =   p;
    char_type *const                end         =   q;
    char_type                       *last_src   =   begin;
    char_type                       *last_dest  =   env_begin;

    // This loop does two things in one go (for efficiency).
    //
    // Firstly, it copies the source (which is read-only) to the m_environment
    // buffer.
    //
    // Secondly, it processes the source, and adjusts the m_symbols and
    // m_environment contents accordingly.
    for(; begin != end;)
    {
        *env_begin = *begin;

        if(*begin == 0)
        {
            sym_begin->name  =   last_dest;
            for(; last_src != begin; ++last_src, ++last_dest)
            {
                if(*last_src == '=')
                {
                    *last_dest = '\0';
                    ++last_src;
                    ++last_dest;
                    break;
                }
            }
            sym_begin->value    =   last_dest;
            last_src            =   ++begin;
            last_dest           =   ++env_begin;

            ++sym_begin;
        }
        else
        {
            ++begin;
            ++env_begin;
        }
    }

    winstl_ns_qual_std(sort)(buffer.begin(), buffer.end(), compare_symbol());
}

template <>
inline /* static */ environment_sequence<ws_char_a_t>::char_type const *environment_sequence<ws_char_a_t>::get_environment_strings()
{
    return static_cast<ws_char_a_t const *>(::GetEnvironmentStringsA());
}

template <>
inline /* static */ environment_sequence<ws_char_w_t>::char_type const *environment_sequence<ws_char_w_t>::get_environment_strings()
{
    return static_cast<ws_char_w_t const *>(::GetEnvironmentStringsW());
}

template <>
inline /* static */ void environment_sequence<ws_char_a_t>::free_environment_strings(environment_sequence<ws_char_a_t>::char_type *s)
{
    ::FreeEnvironmentStringsA(s);
}

template <>
inline /* static */ void environment_sequence<ws_char_w_t>::free_environment_strings(environment_sequence<ws_char_w_t>::char_type *s)
{
    ::FreeEnvironmentStringsW(s);
}

template <>
inline /* static */ ws_int_t environment_sequence<ws_char_a_t>::_compare_symbol(environment_sequence<ws_char_a_t>::symbol const &lhs, environment_sequence<ws_char_a_t>::symbol const &rhs)
{
    return lstrcmpiA(lhs.name, rhs.name);
}

template <>
inline /* static */ ws_int_t environment_sequence<ws_char_w_t>::_compare_symbol(environment_sequence<ws_char_w_t>::symbol const &lhs, environment_sequence<ws_char_w_t>::symbol const &rhs)
{
    return lstrcmpiW(lhs.name, rhs.name);
}


template <ws_typename_param_k C>
inline environment_sequence<C>::environment_sequence()
    : m_p(get_environment_strings())
    , m_symbols(_calc_items(m_p, &m_q))
    , m_environment(m_q - m_p)
{
    _prepare_items(m_symbols, m_environment, const_cast<char_type*>(m_p), const_cast<char_type*>(m_q));
}

template <ws_typename_param_k C>
inline environment_sequence<C>::~environment_sequence() winstl_throw_0()
{
    // The documentation for FreeEnvironmentStrings does not explicitly state
    // that it is legal to free a null string, so we must do the test.
    if(0 != m_p)
    {
        free_environment_strings(const_cast<char_type*>(m_p));
    }
}

template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::const_iterator environment_sequence<C>::begin() const
{
    return m_symbols.begin();
}

template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::const_iterator environment_sequence<C>::end() const
{
    return m_symbols.end();
}

template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::const_reverse_iterator environment_sequence<C>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::const_reverse_iterator environment_sequence<C>::rend() const
{
    return const_reverse_iterator(begin());
}


template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::size_type environment_sequence<C>::size() const
{
    return m_symbols.size();
}

template <ws_typename_param_k C>
inline ws_bool_t environment_sequence<C>::empty() const
{
    return size() == 0;
}

template <ws_typename_param_k C>
inline ws_typename_type_k environment_sequence<C>::value_type environment_sequence<C>::operator [](ws_typename_type_k environment_sequence<C>::size_type index) const
{
    winstl_message_assert("index access out of range in environment_sequence", index < size() + 1);   // Has to be +1, since legitimate to take address of one-past-the-end

    return m_symbols.data()[index];
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_ENVIRONMENT_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
