/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_functionals.h
 *
 * Purpose:     A number of useful functionals .
 *
 * Created:     19th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_FUNCTIONALS
#define _WINSTL_INCL_H_WINSTL_FUNCTIONALS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_FUNCTIONALS_MAJOR      1
#define _WINSTL_VER_H_WINSTL_FUNCTIONALS_MINOR      4
#define _WINSTL_VER_H_WINSTL_FUNCTIONALS_REVISION   8
#define _WINSTL_VER_H_WINSTL_FUNCTIONALS_EDIT       26
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_STRING_ACCESS
 #include "stlsoft_string_access.h" // c_str_ptr, etc.
#endif /* !_STLSOFT_INCL_H_STLSOFT_STRING_ACCESS */
#ifndef _WINSTL_INCL_H_WINSTL_STRING_ACCESS
 #include "winstl_string_access.h"  // c_str_ptr, etc.
#endif /* !_WINSTL_INCL_H_WINSTL_STRING_ACCESS */
#ifndef _WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
 #include "winstl_filesystem_traits.h"
#endif /* !_WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */
#ifndef _WINSTL_FUNCTIONALS_NO_STD
 #include <functional>
#endif /* _WINSTL_FUNCTIONALS_NO_STD */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Function object that compares two file-system paths
///
/// \param C The character type
template <ws_typename_param_k C>
struct compare_path
#ifndef _WINSTL_FUNCTIONALS_NO_STD
    : winstl_ns_qual_std(binary_function)<const C *, const C *, ws_bool_t>
#endif /* _WINSTL_FUNCTIONALS_NO_STD */
{
public:
    /// The character type
    typedef C                                                                       char_type;
#ifndef _WINSTL_FUNCTIONALS_NO_STD
private:
    typedef winstl_ns_qual_std(binary_function)<const C *, const C *, ws_bool_t>    parent_class_type;
public:
    /// The first argument type
    typedef ws_typename_type_k parent_class_type::first_argument_type               first_argument_type;
    /// The second argument type
    typedef ws_typename_type_k parent_class_type::second_argument_type              second_argument_type;
    /// The result type
    typedef ws_typename_type_k parent_class_type::result_type                       result_type;
#else
    /// The first argument type
    typedef const char_type                                                         *first_argument_type;
    /// The second argument type
    typedef const char_type                                                         *second_argument_type;
    /// The result type
    typedef ws_bool_t                                                               result_type;
#endif /* _WINSTL_FUNCTIONALS_NO_STD */
    /// The traits type
    typedef filesystem_traits<C>                                                    traits_type;
    /// The current parameterisation of the type
    typedef compare_path<C>                                                         class_type;

public:
    /// Function call, compares \c s1 with \c s2
    ///
    /// \note The comparison is determined by evaluation the full-paths of both \c s1 and \c s2
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    template <ws_typename_param_k T1, ws_typename_param_k T2>
    result_type operator ()(T1 const &s1, T2 const &s2)
    {
        return _compare(c_str_ptr(s1), c_str_ptr(s2));
    }
#else
    result_type operator ()(first_argument_type s1, second_argument_type s2)
    {
        return _compare(s1, s2);
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */

// Implementation
private:
    result_type _compare(char_type const *s1, char_type const *s2)
    {
        char_type   path1[_MAX_PATH + 1];
        char_type   path2[_MAX_PATH + 1];
        char_type   *dummy;
        result_type result;

        if(!traits_type::get_full_path_name(s1, winstl_num_elements(path1), path1, &dummy))
        {
            result = false;
        }
        else if(!traits_type::get_full_path_name(s2, winstl_num_elements(path2), path2, &dummy))
        {
            result = false;
        }
        else
        {
            traits_type::ensure_dir_end(path1);
            traits_type::ensure_dir_end(path2);

            result = traits_type::str_compare_no_case(path1, path2) == 0;
        }

        return result;
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_FUNCTIONALS */

/* ////////////////////////////////////////////////////////////////////////// */
