/* ////////////////////////////////////////////////////////////////////////////
 * File:        winstl_processheap_allocator.h
 *
 * Purpose:     processheap_allocator class.
 *
 * Created:     25th February 2002
 * Updated:     22nd August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
#define _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_MAJOR     1
#define _WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_MINOR     6
#define _WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_REVISION  2
#define _WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_EDIT      31
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_ALLOCATOR_BASE
 #include "stlsoft_allocator_base.h"    // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT_ALLOCATOR_BASE */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_allocator Allocator Library
/// \ingroup libraries
/// \brief This library provides STL-compatible <code><b>allocator</b></code> types

/// \weakgroup winstl_allocator_library Allocator Library (WinSTL)
/// \ingroup WinSTL libraries_allocator
/// \brief This library provides STL-compatible <code><b>allocator</b></code> types for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k T>
class processheap_allocator;

// Specialisation for void
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template<>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
class processheap_allocator<void>
{
public:
    typedef void                                value_type;
    typedef processheap_allocator<void>         class_type;
    typedef void                                *pointer;
    typedef void const                          *const_pointer;
    typedef ptrdiff_t                           difference_type;
    typedef ws_size_t                           size_type;

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    /// The allocator <b><code>rebind</code></b> structure
    template <ws_typename_param_k U>
    struct rebind
    {
        typedef processheap_allocator<U>        other;
    };
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */
};

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/// STL Allocator based on the Win32 process heap
///
/// \param T The value_type of the allocator
template <ws_typename_param_k T>
class processheap_allocator
{
public:
    /// The value type
    typedef T                                   value_type;
    /// The type of the current parameterisation
    typedef processheap_allocator<value_type>   class_type;
    /// The pointer type
    typedef value_type                          *pointer;
    /// The non-mutating (const) pointer type
    typedef value_type const                    *const_pointer;
    /// The reference type
    typedef value_type                          &reference;
    /// The non-mutating (const) reference type
    typedef value_type const                    &const_reference;
    /// The difference type
    typedef ptrdiff_t                           difference_type;
    /// The size type
    typedef ws_size_t                           size_type;
    /// The type used in deallocate()
#ifdef __STLSOFT_CF_ALLOCATOR_TYPED_DEALLOCATE_POINTER
    typedef pointer                             deallocate_pointer;
#else
    typedef void                                *deallocate_pointer;
#endif /* __STLSOFT_COMPILER_IS_MSVC && _MSC_VER == 1200 */

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    /// The allocator <b><code>rebind</code></b> structure
    template <ws_typename_param_k U>
    struct rebind
    {
        typedef processheap_allocator<U>         other;
    };
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */

// Construction
public:
    /// Default constructor
    processheap_allocator() winstl_throw_0()
#ifndef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
        : m_processheap(::GetProcessHeap())
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    {}
#ifndef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    /// Copy constructor
    processheap_allocator(processheap_allocator const &/* rhs */) winstl_throw_0()
#ifndef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
        : m_processheap(::GetProcessHeap())
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    {}
#else
    /// Copy constructor
    template <ws_typename_param_k U>
    processheap_allocator(processheap_allocator<U> const &/* rhs */) winstl_throw_0()
#ifndef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
        : m_processheap(::GetProcessHeap())
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    {}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
    /// Destructor
    ~processheap_allocator() winstl_throw_0()
    {}

// Attributes
public:
    /// Returns the maximum number of allocatable entities
    size_type max_size() const winstl_throw_0()
    {
        return static_cast<size_type>(-1) / sizeof(value_type);
    }

// Conversion
public:
    /// Returns the address corresponding to the given reference
    ///
    /// \param x A reference to a \c value_type instance whose address will be calculated
    pointer address(reference x) const winstl_throw_0()
    {
        return &x;
    }
    /// Returns the address corresponding to the given non-mutable (const) reference
    ///
    /// \param x A non-mutable (const) reference to a \c value_type instance whose address will be calculated
    const_pointer address(const_reference x) const winstl_throw_0()
    {
        return &x;
    }

// Allocation
public:
    /// Allocates a block of memory sufficient to store \c n elements of type \c value_type
    ///
    /// \param n The number of elements to allocate
    /// \param pv A hint to enhance localisation (not used in this class)
    /// \return The allocated block, or the null pointer (if the allocation fails and the translation does not support throwing exceptions upon memory exhaustion)
#ifdef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
    static
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    pointer allocate(size_type n, processheap_allocator<void>::const_pointer = NULL)
    {
        processheap_allocator<void>::pointer p   =   static_cast<processheap_allocator<void>::pointer>(::HeapAlloc(_get_process(), 0, n * sizeof(value_type)));

#ifdef __STLSOFT_CF_THROW_BAD_ALLOC
        if(p == NULL)
        {
            throw ::std::bad_alloc();
        }
#endif /* __STLSOFT_CF_THROW_BAD_ALLOC */

        return static_cast<pointer>(p);
    }

#ifdef __STLSOFT_CF_ALLOCATOR_CHARALLOC_METHOD
    /// The type used in deallocate
    char *_Charalloc(size_type n)
    {
        return reinterpret_cast<char*>(allocate(n, NULL));
    }
#endif /* __STLSOFT_CF_ALLOCATOR_CHARALLOC_METHOD */

    /// Deallocates a pointer
    ///
    /// \param p The memory block to deallocate
    /// \param n The number of blocks to deallocate
#ifdef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
    static
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    void deallocate(pointer p, size_type)
    {
        ::HeapFree(_get_process(), 0, p);
    }

    /// Deallocates a pointer
    ///
    /// \param p The memory block to deallocate
#ifdef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
    static
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */
    void deallocate(pointer p)
    {
        ::HeapFree(_get_process(), 0, p);
    }

// Operations
public:
    /// In-place constructs an instance of the \c value_type, with the given value
    ///
    /// \param p The location in which to construct the instance
    /// \param x The value with which to copy construct the instance
    void construct(pointer p, value_type const &x)
    {
        winstl_assert(p != NULL);

        new(p) value_type(x);
    }

    /// In-place constructs an instance of the \c value_type
    ///
    /// \param p The location in which to construct the instance
    void construct(pointer p)
    {
        winstl_assert(p != NULL);

        new(p) value_type();
    }

    /// In-place destroys an instance
    ///
    /// \param p The instance whose destructor is to be called
    void destroy(pointer p) winstl_throw_0()
    {
        winstl_assert(p != NULL);

        winstl_destroy_instance(T, value_type, p);
    }

// Members
protected:
#ifdef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
    static HANDLE _get_process()
    {
        static HANDLE   s_processheap   =   ::GetProcessHeap();

        return s_processheap;
    }
#else
    HANDLE _get_process()
    {
        return m_processheap;
    }
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */

// Members
protected:
#ifndef __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS
    HANDLE  m_processheap;
#endif /* __STLSOFT_CF_ALLOCATOR_STATIC_ALLOCATE_METHODS */

// Not to be implemented
private:
    class_type const &operator =(class_type const &rhs);
};

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k T>
inline ws_bool_t operator ==(processheap_allocator<T> const &/* lhs */, processheap_allocator<T> const &/* rhs */)
{
    return ws_true_v;
}

template <ws_typename_param_k T>
inline ws_bool_t operator !=(processheap_allocator<T> const &/* lhs */, processheap_allocator<T> const &/* rhs */)
{
    return ws_false_v;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_allocator_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */

/* ////////////////////////////////////////////////////////////////////////// */
